/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QArrangement
 * 2006-11-16
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************/

# ifndef QARRANGEMENT_H
# define QARRANGEMENT_H

# include <QList>
# include <QString>
# include <QThread>
# include <QMutex>
# include <QWaitCondition>

# include <QAxel/QAxelObject.h>
# include <QAxel/QPoint3f.h>

# include <QData/QCell.h>
# include <QData/QRegionVertex.h>
# include <QData/QRegionEdge.h>
# include <QData/QRegionFace.h>
# include <QData/QRegion3f.h>
# include <QData/QGraph.h>
# include <QData/QGraphNode.h>
# include <QData/QGraphEdge.h>
# include <QData/QAugmentedInfluenceGraph.h>
# include <QData/QTreeNode.h>
# include <QData/QOctree.h>

# include <QGui/MainWindow.h>
# include <QGui/Viewer.h>

using namespace QAxel ;
using namespace QData ;

namespace QArrangement {

typedef QAxelObject Object_t ;
typedef QPoint3f Point_t ;
typedef QRegionVertex Vertex_t ;
typedef QRegionEdge Edge_t ;
typedef QRegionFace Face_t ;
typedef QRegion3f Region_t ;
typedef QCell Cell_t ;
typedef QTreeNode<Region_t *, Cell_t *> Node_t ;
typedef QOctree<Region_t *, Cell_t *> Octree_t ;
typedef QGraph Graph_t ;
typedef QGraphNode GraphNode_t ;

/****************************************************************************/
/*!
  \class QArrangement QArrangement.h
  \brief The class QArrangement is an abstract base class for arrangement
  algorithms.

  This class contains generic arrangement procedures that mainly corresponds
  to algorithmic geometry computations which are not related to a specific
  kind of object.
*/
/****************************************************************************/
// class QArrangement : public QThread
class QArrangement : public QObject
{
    Q_OBJECT
	
public:
    QArrangement(QString type, double epsilon = 1e-1, double epsilon_max = 1.0) ;
    ~QArrangement(void) ;

signals:
    void display(QGraph *) ;
  
public slots:
    virtual void play(void) ;
    virtual void pause(void) ;
    virtual void stop(void) ;
    virtual void init(QList<Object_t *> objects) ;
    virtual void run(void) ;
    void setTree(QAxelObject * o) ;
    void setGraphicalFusionVerbosity(bool v) { gverbose_fusion = v ; }
    void setGraphicalCellsVerbosity(bool v) { gverbose_cells = v ; }
	
public slots:
    virtual void addObject(QAxelObject * o) ;
    virtual void removeObject(Object_t * o) ;
    virtual QList<Region_t *> getRegions(Object_t * o) ;

protected:
    virtual bool regular(Object_t * o, Node_t * n) = 0 ;
    virtual QList<Region_t *> topology(Object_t * o, Node_t * n) = 0 ;
    virtual QList<Cell_t *> subdivide(Object_t * o, Node_t * n) = 0 ;

protected:
    virtual QList<Region_t *> fusion(Octree_t octree) ;
    virtual QList<Region_t *> fusion(Node_t * node) ;
    virtual QList<Region_t *> fusion(QList<Region_t *> & regions) ;
    virtual Region_t * dummy(Node_t * n) ;

// generic functions related to the computation of regions for several objects
public slots:
    virtual QList<GraphNode_t *> locate(Point_t * p) ;
    virtual QList<GraphNode_t *> locate(Point_t * p, GraphNode_t * n) ;

protected:
    virtual QList<GraphNode_t *> locate(Region_t * r) ;
    virtual QList<GraphNode_t *> locate(Region_t * r, GraphNode_t * n) ;
    
protected:
    virtual Vertex_t * xsmallest(QList<QRegionVertex *>& l) ;
    virtual Vertex_t * xgreatest(QList<QRegionVertex *>& l) ;
    virtual Vertex_t * ysmallest(QList<QRegionVertex *>& l) ;
    virtual Vertex_t * ygreatest(QList<QRegionVertex *>& l) ;
    virtual bool isCellBorder(Vertex_t * p, Cell_t * bb) ;
    virtual QList<Vertex_t *> clockwiseSubdivision(QList<Point_t *> & intersectionPoints, Cell_t * bb, Vertex_t * nw, Vertex_t * ne, Vertex_t * se, Vertex_t * sw) ;
	
public:
    QGraph * graph(void) ;

protected:
    enum SubdivisionPolicy { 
	Center
    } ;
    
protected:
    QAugmentedInfluenceGraph * a_graph ;
    QGraphNode * a_root ;
    SubdivisionPolicy a_policy ;
    QMap<Object_t *, Graph_t *> a_trees ;
	
private:
    QString a_type ;
    bool m_stop ;
    bool m_pause ;
    QWaitCondition m_pauseCondition ;
    QMutex m_mutex ;

private:
    double c_epsilon ;
    double c_epsilon_max ;

protected:
    MainWindow * mainwindow ;
    bool gverbose_cells ;
    bool gverbose_fusion ;
} ;

} ; // namespace QArrangement ;

# endif // QARRANGEMENT_H
