/*****************************************************************************
 *
 ****************************************************************************/

# include <QAxel/QDisplayList.h>

namespace QAxel {

/****************************************************************************/
/*! Creates the instance. Does nothing special, just passed arguments to
    \link Node Node \endlink.
*/
/****************************************************************************/
QDisplayList::QDisplayList(QAxelObject * parent) : QAxelObject (parent) 
{  
    // by default we're not recording and thelist is invalid
    m_Recording = false;
    m_Valid     = false;
}
    

/****************************************************************************/
/*! Deletes the list. This method is provided so that the list can be 
    deallocated before deleting.
*/
/****************************************************************************/
QDisplayList::~QDisplayList () 
{   
    // do we have a list at all?
    if ( isValid () || isRecording () ) {
	// delete the list
	glDeleteLists ( m_ListId, 1 );
    }
}


/****************************************************************************/
/*! Renders the list. The call is ignored if the list is currently being
    recorded or is not valid.
*/
/****************************************************************************/
void QDisplayList::draw() 
{
    // is the list ready to be rendered?
    if ( ! isValid () && ! isRecording () ) {
	// oops, can't render this list, find out more exactly what's wrong
	if ( ! isValid () ) {
	    // list is invalid, maybe not recorded at all?
	    printf ("QDisplayList::render - list is invalid");
	    return;
	}
	
	else {
	    // list is still being recorded!
	    printf ("QDisplayList::render - list is still being recorde");
	    return;
	}
    }
    
    // all is ok, so render the list
    glCallList ( m_ListId );
}
    
void QDisplayList::drawWithNames()
{
    ;
}

/****************************************************************************/
/*! Starts recording a list. IF we already have a valid list the old list is
    cleared first. If we're currently recording nothing is done, but a
	warning is printed.
*/
/****************************************************************************/
void QDisplayList::begin () 
{  
    // are we recording?
    if ( isRecording () ) {
	// oh, were currently not recording, print error
	printf ("QDisplayList::begin - already recording a list!");
	return;
    }
    
    // do we already have a list?
    if ( isValid () ) {
	// ah, so delete it first
	glDeleteLists ( m_ListId, 1 );
    }
    
    // create the new list
    if ( ( m_ListId = glGenLists ( 1 ) ) == 0 ) {
	// oops, could not allocated the new list
	printf ("QDisplayList::begin - could not allocate a list!");
	return;
    }
    
    // start recording the list
    glNewList ( m_ListId, GL_COMPILE );
    
    // set the flags
    m_Valid     = false;
    m_Recording = true;
}


/****************************************************************************/
/*! Finishes recording a list if we're currently recording, otherwise does
    nothing.
*/
/****************************************************************************/
void QDisplayList::end () 
{   
    // are we recording?
    if ( ! isRecording () ) {
	// oh, were currently not recording, print error
	printf ("QDisplayList::end - not currently recording a list!");
    }
    
    else {
	// finish recording
	glEndList ();
	
	// now the list is valid
	m_Valid     = true;
	m_Recording = false;
    }
}


/****************************************************************************/
/*! Returns \c true if the list is valid, i.e. something has been 
    successfully recorded and the list is not currebtly being recorded.
*/
/****************************************************************************/
bool QDisplayList::isValid () const 
{  
    // return the validness
    return m_Valid;
}


/****************************************************************************/
/*! Returns the current state of the recording. Returns \c true if the list
    is currently being recorded, and \c false if not. It is illegal to leave
	the list in a recording state.
*/
/****************************************************************************/
bool QDisplayList::isRecording () const 
{  
    // return the state
    return m_Recording;
}

void QDisplayList::invalidate() 
{
    m_Valid = false ;
}

} ; // namespace QAxel
