# include <QAxel/QVertex.h>
# include <QAxel/QEdge.h>
# include <QAxel/QFace.h>
# include <QAxel/QMesh.h>

using namespace qglviewer ;

namespace QAxel {       
	
/*!
 * This class allows to handle frame manipulation forwarding over incident vertices
 */
class FaceFrameSetConstraint : public Constraint 
{
public:
  	void clearSet();
  	void addObjectToSet(QAxel::QAxelObject* o);
               
	QMesh * m ;
	QFace * f ;

  	virtual void constrainTranslation(Vec &translation, Frame *const) {
		for (int i=0 ; i<f->indexes_.size() ; i++) m->vertices_[f->indexes_[i]]->frame()->translate(translation);
	}
  	
	virtual void constrainRotation(Quaternion &rotation, Frame *const frame) {
		// A little bit of math. Easy to understand, hard to guess (tm).
  		// rotation is expressed in the frame local coordinates system. Convert it back to world coordinates.
  		const Vec worldAxis = frame->inverseTransformOf(rotation.axis());
  		const Vec pos = frame->position();
  		const float angle = rotation.angle();

    		for (int i=0 ; i<f->indexes_.size() ; i++) {
			// Rotation has to be expressed in the object local coordinates system.
			Quaternion qObject(m->vertices_[f->indexes_[i]]->frame()->transformOf(worldAxis), angle);
			m->vertices_[f->indexes_[i]]->frame()->rotate(qObject);
		
			// Comment these lines only rotate the objects
			Quaternion qWorld(worldAxis, angle);
			// Rotation around frame world position (pos)
			m->vertices_[f->indexes_[i]]->frame()->setPosition(pos + qWorld.rotate(m->vertices_[f->indexes_[i]]->frame()->position() - pos));
      		}                                                   
	}
} ;  

int QFace::id_ = 0 ;         

QFace::QFace(int index, QMesh * m) 
{
	this->m = m ;
	this->index_ = index ;
	
	QString str ;
	setProperty("type", "Face") ;
	setProperty("name", "Face"+str.setNum(id_)) ;
	id_++ ;
	
	setObjectColor(new QColor(160,160,160)) ;   
	          
	FaceFrameSetConstraint * c = new FaceFrameSetConstraint ;
	c->m = m ; 
	c->f = this ;
	frame_->setConstraint(c) ;    
}       

void QFace::push_index(int index) 
{
	m->glindexes_[3*index_+indexes_.size()] = index ;
	indexes_ << index ;
}                           

void QFace::setColor(float r, float g, float b) 
{
	int ir = (int)(r*255) ;
	int ig = (int)(g*255) ;
	int ib = (int)(b*255) ;
	setObjectColor(new QColor(ir,ig,ib)) ;
}                                         

void QFace::setColor(float r, float g, float b, float a) 
{
	int ir = (int)(r*255) ;
	int ig = (int)(g*255) ;
	int ib = (int)(b*255) ;     
	int ia = (int)(a*255) ;
	setObjectColor(new QColor(ir,ig,ib,ia)) ;
}

Vec QFace::computeNormal(void) 
{
	QVertex * v1 = m->vertices_[indexes_[0]] ;
	QVertex * v2 = m->vertices_[indexes_[1]] ;
	QVertex * v3 = m->vertices_[indexes_.last()] ;
	Vec ve1(v2->x()-v1->x(), v2->y()-v1->y(), v2->z()-v1->z()) ;
	Vec ve2(v3->x()-v1->x(), v3->y()-v1->y(), v3->z()-v1->z()) ;                                    
	normal_ = ve1^ve2 ;
	normal_.normalize() ;                                    
	return normal_ ;
}
 
void QFace::draw(void) 
{
	glPushMatrix() ;
		if(m->mode() == QMesh::Edit) glColor() ;
		glBegin(GL_POLYGON) ;
		glNormal3f(normal_.x, normal_.y, normal_.z) ;
		for(int i=0 ; i<indexes_.size() ; i++) 
		glVertex3f(m->vertices_[indexes_[i]]->x(), m->vertices_[indexes_[i]]->y(), m->vertices_[indexes_[i]]->z()) ;
		glEnd() ;
	glPopMatrix() ;
}

void QFace::drawWithNames(void)
{
	glPushName((GLuint)this);
	draw();
	glPopName();
}

QString QFace::toString(void)
{
	QString text ; QString tmp ;
	text += name() + "(" ;
	foreach(int i, indexes_) text += tmp.setNum(i) + ", " ;
	text +=  ")" ;
	return text ;
}

std::ostream& operator <<(std::ostream& stream, QFace & f) 
{
	return stream << f.toString().toLatin1().data() ; 
}

std::ostream& operator <<(std::ostream& stream, QFace * f) 
{
	return stream << f->toString().toLatin1().data() ; 
}

} ; // namespace QAxel
