/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QMaterial
 * 2007-06-17
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************/

# include <QAxel/QMaterial.h>

namespace QAxel {

/****************************************************************************/
/*! Creates an default material with default values for all components.
 */
/****************************************************************************/
QMaterial::QMaterial (QAxelObject * parent) : QAxelObject(parent)
{
    m_Ambient  = QColor ( 40, 40, 40 ) ;
    m_Diffuse  = QColor ( 200, 200, 200 ) ;
    m_Specular = QColor ( 0, 0, 0 ) ;
    m_Emissive = QColor ( 0, 0, 0 ) ;
    m_Shininess = 0.2 ;
}

/****************************************************************************/
/*! Creates a material from the passed colors and components. The material
    is an exact copy of the arguments. Deep copies are made of all colors.

	\arg const QColor & Ambient - ambient color for the material.
	\arg const QColor & Diffuse - diffuse color for the material.
	\arg const QColor & Specular  - specular color for the material.
	\arg const QColor & Emissive - emissive color for the material.
	\arg const float Shininess - shininess for the material.
	\arg const float Transparency - shininess for the material.
 */
/****************************************************************************/
QMaterial::QMaterial(QAxelObject * parent, 
		     const QColor & ambient, 
		     const QColor & diffuse,
		     const QColor & specular, 
		     const QColor & emissive,
		     const float shininess) : QAxelObject(parent) 
{
    Q_ASSERT(shininess >= 0 && shininess <= 1.0) ;

    m_Ambient  = ambient ;
    m_Diffuse  = diffuse ;
    m_Specular = specular ;
    m_Emissive = emissive ;

    m_Shininess = shininess ;
}

/****************************************************************************/
/*! Creates a material by copying the passed material \a m. A deep copy is
    made from all components.

    \arg const QMaterial & m - material to copy.
 */
/****************************************************************************/
QMaterial::QMaterial(QAxelObject * Parent, const QMaterial & m) : QAxelObject(Parent)
{
    m_Ambient  = m.ambient() ;
    m_Diffuse  = m.diffuse() ;
    m_Specular = m.specular() ;
    m_Emissive = m.emissive() ;
    
    m_Shininess = m.shininess() ;
}

/****************************************************************************/
/*! Renders the material. This means that the current material in the
    OpenGL-context is changed to this material.
 */
/****************************************************************************/
void QMaterial::draw(void) 
{
    // set the 'one-element' component, i.e. shininess
    glMaterialf ( GL_FRONT, GL_SHININESS, m_Shininess );

    // set the colors
    qreal * ambient  = 0 ; m_Ambient.getRgbF(&ambient[0],   &ambient[1],  &ambient[2],  &ambient[3]) ;
    qreal * diffuse  = 0 ; m_Diffuse.getRgbF(&diffuse[0],   &diffuse[1],  &diffuse[2],  &diffuse[3]) ;
    qreal * specular = 0 ; m_Specular.getRgbF(&specular[0], &specular[1], &specular[2], &specular[3]) ;
    qreal * emission = 0 ; m_Emissive.getRgbF(&emission[0], &emission[1], &emission[2], &emission[3]) ;
    glMaterialfv( GL_FRONT, GL_AMBIENT,  (GLfloat *)ambient  ) ;
    glMaterialfv( GL_FRONT, GL_DIFFUSE,  (GLfloat *)diffuse  ) ;
    glMaterialfv( GL_FRONT, GL_SPECULAR, (GLfloat *)specular ) ;
    glMaterialfv( GL_FRONT, GL_EMISSION, (GLfloat *)emission ) ;
}

void QMaterial::drawWithNames(void) 
{

}

/****************************************************************************/
/*! Returns the \e part of the material.
 */
/****************************************************************************/
const QColor & QMaterial::ambient(void) const 
{
    return m_Ambient ;
}

/****************************************************************************/
/*! Returns the \e diffuse part of the material.
 */
/****************************************************************************/
const QColor & QMaterial::diffuse(void) const 
{
    return m_Diffuse ;
}

/****************************************************************************/
/*! Returns the \e specular part of the material.
 */
/****************************************************************************/
const QColor & QMaterial::specular(void) const 
{
    return m_Specular ;
}

/****************************************************************************/
/*! Returns the \e emissive part of the material.
 */
/****************************************************************************/
const QColor & QMaterial::emissive(void) const 
{
    return m_Emissive ;
}

/****************************************************************************/
/*! Returns the \e shininess of the material.
 */
/****************************************************************************/
float QMaterial::shininess(void) const 
{
    return m_Shininess ;
}

/****************************************************************************/
/*! Sets a new \e color for the material.

    \arg const QColor & ambient - new  component.
 */
/****************************************************************************/
void QMaterial::setAmbient(const QColor & ambient) 
{
    m_Ambient = ambient ;
}

/****************************************************************************/
/*! Sets a new \e diffuse color for the material.

    \arg const QColor & diffuse - new diffuse component.
 */
/****************************************************************************/
void QMaterial::setDiffuse(const QColor & diffuse) 
{
    m_Diffuse = diffuse ;
}

/****************************************************************************/
/*! Sets a new \e specular color for the material.

    \arg const QColor & specular - new specular component.
 */
/****************************************************************************/
void QMaterial::setSpecular(const QColor & specular) 
{
    m_Specular = specular ;
}

/****************************************************************************/
/*! Sets a new \e emissive color for the material.

    \arg const QColor & emissive - new emissive component.
 */
/****************************************************************************/
void QMaterial::setEmissive(const QColor & emissive) 
{
    m_Emissive = emissive ;
}

/****************************************************************************/
/*! Sets a new \e color for the material.  */
/****************************************************************************/
void QMaterial::setShininess(const float shininess) 
{
    m_Shininess = shininess ;
}

/****************************************************************************/
/*! Overloaded \e =-operator to provide assignment from one material to
    another. A deep copy is made from the source-material.

    \arg const QMaterial & m - material to assign from.
 */
/****************************************************************************/
QMaterial & QMaterial::operator = (const QMaterial & m) 
{
    m_Ambient  = m.ambient() ;
    m_Diffuse  = m.diffuse() ;
    m_Specular = m.specular() ;
    m_Emissive = m.emissive() ;

    m_Shininess = m.shininess() ;

    return *this ;
}

} ; // namespace QAxel
