/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QVertexArray
 * 2007-06-16
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# include <QAxel/QVertexArray.h>
 
namespace QAxel {

/****************************************************************************/
/*! Creates the node and assigns the passed values to members.

    \arg QAxelObject * parent - parent for this node
    \arg const Type dataType - the type of polygin to be rendered.
    \arg const int size - number of vertices. Must be the total number of
    vertices contained in \a data.
*/
/****************************************************************************/
QVertexArray::QVertexArray(QAxelObject * parent, const QVertexArray::Type dataType, const int size) : QAxelObject(parent)
{  
    m_Type     = dataType ;
    m_Mode     = PerFace ;
    m_Vertices = 0 ;
    m_Normals  = 0 ;
    m_Colors   = 0 ;
    m_Size     = size ;

    m_OwnData = false ;
    m_Valid = false ;
  
    if(m_ArrayCount == 0) {
	glEnableClientState(GL_VERTEX_ARRAY) ;
	if(m_Vertices != 0)
	    glEnable(GL_VERTEX_ARRAY) ;

	if(m_Colors != 0)
	    glEnable(GL_COLOR_ARRAY) ;

	if(m_Normals != 0)
	    glEnable(GL_NORMAL_ARRAY) ;
    }
    
    m_ArrayCount++ ;
}

/****************************************************************************/
/*! Creates an instance of the class with no data.

    \arg QAxelObject * parent - the desired parent for the node.
    \arg const Type dataType - the default datatype.
*/
/****************************************************************************/
QVertexArray::QVertexArray(QAxelObject * parent, const Type dataType) : QAxelObject(parent) 
{
    m_Type = dataType ;
    m_Mode = PerFace ;
    
    m_OwnData  = false ;
    m_Vertices = 0 ;
    m_Normals  = 0 ;
    m_Colors   = 0 ;
    m_Size     = 0 ;
  
    m_Valid = false ;

    if(m_ArrayCount == 0) {
	glEnableClientState(GL_VERTEX_ARRAY) ;
	if(m_Vertices != 0)
	    glEnable(GL_VERTEX_ARRAY) ;

	if(m_Colors != 0)
	  glEnable(GL_COLOR_ARRAY) ;

	if(m_Normals != 0)
	    glEnable(GL_NORMAL_ARRAY) ;
    }
  
    m_ArrayCount++ ;
}

/****************************************************************************/
/*! Creates an instance of the class with passed data.

    \arg QAxelObject * parent - the desired parent for the node.
    \arg const int size = 0 - number of elements to render.
    \arg const Type dataType - the default datatype.
    \arg float * vertices = 0 - pointer to the vertexes.
    \arg float * colors = 0 - pointer to the colors.
    \arg float * normals = 0 - pointer to the normals.
    \arg const NormalMode Mode = PerFace - define how normals are assigned.
*/
/****************************************************************************/
QVertexArray::QVertexArray(QAxelObject * parent, const int size, const Type DataType, 
		    float * vertices, float * colors, float * normals, const NormalMode Mode) 
    : QAxelObject(parent) 
{
    m_Type     = DataType ;
    m_Mode     = Mode ;
    m_Size     = size ;
    m_Vertices = vertices ;
    m_Colors   = colors ;
    m_Normals  = normals ;
  
    m_Valid = false ;

    if(m_ArrayCount == 0) {
	glEnableClientState(GL_VERTEX_ARRAY) ;
	if(m_Vertices != 0)
	    glEnableClientState(GL_VERTEX_ARRAY) ;

	if(m_Colors != 0)
	    glEnableClientState(GL_COLOR_ARRAY) ;

	if(m_Normals != 0)
	    glEnableClientState (GL_NORMAL_ARRAY) ;
    }

    m_ArrayCount++ ;
}

/****************************************************************************
 * Define the static number of defined arrays. By default there are no
 * define arrays.
 ****************************************************************************/
int QVertexArray::m_ArrayCount = 0 ;

/****************************************************************************/
/*! Destroys the instance. Disables vertex-arrays if this is the last 
    instance.
*/
/****************************************************************************/
QVertexArray::~QVertexArray(void) 
{
    m_ArrayCount-- ;
  
    if(m_ArrayCount == 0) {
	glDisableClientState(GL_VERTEX_ARRAY) ;
	glDisableClientState(GL_COLOR_ARRAY) ;
	glDisableClientState(GL_NORMAL_ARRAY) ;
    }
}

/****************************************************************************/
/*! Renders the vertex-array. Does nothing if no data has been given. Sets
    all arrays that have been passed.
*/
/****************************************************************************/
void QVertexArray::draw(void)
{
    if(m_Type == Undefined || m_Vertices == 0 || m_Size == 0) {
	printf("QVertexArray::render - no data or no type\n") ;
	return ;
    }
  
    if(m_Vertices != 0) {
	glVertexPointer(3, GL_FLOAT, 0, m_Vertices) ;
	glEnableClientState(GL_VERTEX_ARRAY) ;
    } else {
	glDisableClientState(GL_VERTEX_ARRAY) ;
    }

  
    if(m_Colors != 0) {
	glColorPointer( 4, GL_FLOAT, 0, m_Colors) ;
	glEnable(GL_COLOR_ARRAY) ;
    } else {
	glDisableClientState(GL_COLOR_ARRAY) ;
    }

    if(m_Normals != 0) {
	glNormalPointer(GL_FLOAT, 0, m_Normals) ;
	glEnableClientState(GL_NORMAL_ARRAY) ;
    } else {
	glDisableClientState(GL_NORMAL_ARRAY) ;
    }

    glDrawArrays((GLenum)m_Type, 0, m_Size) ;
}

void QVertexArray::drawWithNames(void)
{

}

/****************************************************************************/
/*! Invalidates the datapointer and causes the method to resend the vertex-
    data to OpenGL the next time a rendering occurs.
*/
/****************************************************************************/
void QVertexArray::invalidateData(void)
{
    m_Valid = false ;
}

/****************************************************************************/
/*! Sets the new type for the rendering. The vertices are not changed, but
    used straight 'as is'.

    \arg const Type QVertexArrayType - the new type
*/
/****************************************************************************/
void QVertexArray::setType(const Type type)
{
    m_Type = type ;
}

/****************************************************************************/
/*! Returns the current \e type used for the rendering.
*/
/****************************************************************************/
QVertexArray::Type QVertexArray::type(void) const 
{
    return m_Type ;
}

/****************************************************************************/
/*! Sets a vertex at position \a Index. \a Index must be in the interval
    [0..size()-1] i order to be valid. The old value is overwritten.

    \arg const int index - index of desired vertex.
    \arg const QVertex * new - actual vertex.
*/
/****************************************************************************/
void QVertexArray::setVertex(const int index, QVertex * vertex)
{  
    Q_ASSERT(index >= 0 && index < m_Size) ;
  
    if(index >= 0 && index < m_Size) {
	float * offset = m_Vertices + index * 3 ;
	*(offset + 0) = vertex->x() ;
	*(offset + 1) = vertex->y() ;
	*(offset + 2) = vertex->z() ;
	m_Valid = false ;
    }
}

/****************************************************************************/
/*! Sets a vertex at position \a Index. \a Index must be in the interval
    [0..size()-1] i order to be valid. The old value is overwritten. 

    \arg const int index - index of desired vertex.
	\arg const float x - new x-coordinate.
	\arg const float y - new y-coordinate.
	\arg const float z - new z-coordinate.
*/
/****************************************************************************/
void QVertexArray::setVertex (const int index, const float x, const float y, const float z) 
{  
    Q_ASSERT(index >= 0 && index < m_Size) ;
  
    if(index >= 0 && index < m_Size) {
	float * offset = m_Vertices + index * 3 ;
	*(offset + 0) = x ;
	*(offset + 1) = y ;
	*(offset + 2) = z ;
	m_Valid = false ;
    }
}

/****************************************************************************/
/*! Assigns a new array of predefined values as the new data for the
    rendering. The new data will not be touched in any way, but the
    old data will de deleted if we own it. The new \a Size must be
    larger than \c 0, and the \a Data may not be \c 0.

    \arg const int size - new size.
    \arg float * data - new data. Must not be \c 0.
*/
/****************************************************************************/
void QVertexArray::setVertices(const int size, float * vertices) 
{
    if(size <= 0)
	qDebug("QVertexArray::setQVertexes - %d is an invalid size", size) ;

    Q_ASSERT(vertices != 0) ;
    
    if(m_Vertices != 0 && m_OwnData)
	delete [] m_Vertices ;

    m_Size     = size ;
    m_Vertices = vertices ;
    m_OwnData  = false ;

    m_Valid = false ;
}

/****************************************************************************/
/*! Returns a vertex located at \a index. If \a Index is out of bonds
    nothing is done. the data is put into the pointers pointed to by
    \a x, \a y and \a z. \a Index must be in the interval
    [0..size()-1].

    \arg const int index - index of desired vertex.
    \arg float * x - pointer fo holding the x-coordinate.
    \arg float * y - pointer fo holding the y-coordinate.
    \arg float * z - pointer fo holding the z-coordinate.
*/
/****************************************************************************/
void QVertexArray::vertex (const int index, float * x, float * y, float * z) const 
{
    Q_ASSERT(index >= 0 && index < m_Size) ;
  
    if(index >= 0 && index < m_Size) {
	float * offset = m_Vertices + index * 3 ;
	*x = *offset + 0 ;
	*y = *offset + 1 ;
	*z = *offset + 2 ;
    }
}

/****************************************************************************/
/*! Returns the raw array of vertex-data. It is of the form X1,Y1,Z1,X2,Y2...
    The values are tightly packed floats.
*/
/****************************************************************************/
float * QVertexArray::vertices(void) 
{
    return m_Vertices ;
}

/****************************************************************************/
/*! Sets a color at position \a Index. \a Index must be in the interval
    [0..size()-1] i order to be valid. The old value is overwritten.

    \arg const int index - index of desired vertex.
    \arg  const QColor * vertex - the new color.
*/
/****************************************************************************/
void QVertexArray::setColor (const int index, const QColor & color) 
{
    Q_ASSERT(index >= 0 && index < m_Size) ;
  
    if(index >= 0 && index < m_Size) {
	float * offset = m_Colors + index * 4 ;
	*(offset + 0) = color.red() ;
	*(offset + 1) = color.green() ;
	*(offset + 2) = color.blue() ;
	*(offset + 3) = color.alpha() ;
	m_Valid = false ;
    }
}

/****************************************************************************/
/*! Sets a color at position \a index. \a Index must be in the
    interval [0..size()-1] i order to be valid. The old value is
    overwritten.  Overloaded from the above method as to what
    arguemnts are accepted.

    \arg const int index - index of desired vertex.
    \arg const float red - new red component.
    \arg const float green - new green component.
    \arg const float blue - new blue component.
    \arg const float alpha - new alpha component.
*/
/****************************************************************************/
void QVertexArray::setColor(const int index, const float red, const float green, const float blue, const float alpha) 
{
    Q_ASSERT(index >= 0 && index < m_Size) ;
  
    if(index >= 0 && index < m_Size) {
	float * offset = m_Colors + index * 4 ;
	*(offset + 0) = red ;
	*(offset + 1) = green ;
	*(offset + 2) = blue ;
	*(offset + 3) = alpha ;
	m_Valid = false ;
    }
}

/****************************************************************************/
/*! Assigns a new array of predefined values as the new data for the
    colors. The new data will not be touched in any way, but the old
    data will de deleted if we own it. The new \a Size must be larger
    than \c 0, and the \a QColors may not be \c 0.

    \arg const int size - new size.
    \arg float * colors - new data. Must not be \c 0.
*/
/****************************************************************************/
void QVertexArray::setColors(const int size, float * colors) 
{
    if(size <= 0) 
	printf("QVertexArray::setQColors - %d is an invalid size", size) ;

    Q_ASSERT(colors != 0) ;
 
    if ( m_Colors != 0 && m_OwnData)
	delete [] m_Colors ;
  
    m_Size    = size ;
    m_Colors  = colors ;
    m_OwnData = false ;
    m_Valid = false ;
}

/****************************************************************************/
/*! Returns a \link QColor color \endlink pointed to by \a index. \a Index
    must be in the interval [0..size()-1].

    \arg const int index - index of desired color.
*/
/****************************************************************************/
const QColor QVertexArray::color(const int index) const 
{
    QColor c ;
  
    Q_ASSERT(index >= 0 && index < m_Size) ;
  
    if(index >= 0 && index < m_Size) {
	float * offset = m_Colors + index * 4 ;
	c = QColor(0,0,0) ;
	c.setRgbF(*offset + 0, *offset + 1, *offset + 2, *offset + 3) ;
    } else {
	c = QColor( 0, 0, 0) ;
    }
  
    return c ;
}

/****************************************************************************/
/*! Returns the raw array of color-data. It is of the form RGBRGBRGB....
    The values are tightly packed floats.
*/
/****************************************************************************/
float * QVertexArray::colors(void) 
{
    return m_Colors ;
}

/****************************************************************************/
/*! Returns the number of vertices the vertex-array contains.
*/
/****************************************************************************/
int QVertexArray::size(void) const 
{
    return m_Size ;
}

/****************************************************************************/
/*! Sets a size for the number of vertices. This method should be
    called if no data/size was given in the constructor. This method
    will allocate a chunk of data that can hold \a Size vertices, and
    copy all possible old values.

	If the new size is smaller than than the old, only as many as
	possible are copied, the others are left out. The old data is
	newer deleted if it was assigned as an argument, i.e. the
	emthod will never destroy data it does not own.

	If the new size is larger the new vertices will be undefined,
	and should be filled-in by the user before rendering.

    \arg const int size - new desired size.
*/
/****************************************************************************/
void QVertexArray::setSize(const int size) 
{
    if(size <= 0)
	printf("QVertexArray::setSize - %d is an invalid size", size) ;

    float * New = new float[size * 3] ;
  
    Q_ASSERT(New != 0) ;
 
    int min = m_Size < size ? m_Size : size ;
  
    memcpy(New, m_Vertices, min * 3 * sizeof(float)) ;
  
    if(m_Vertices != 0 && m_OwnData) 
	delete [] m_Vertices ;
  
    m_Size = size ;
    m_Vertices = New ;
    m_OwnData = true ;
    m_Valid = false ;
}

} ; // namespace QAxel
