/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QVertexArray
 * 2007-06-16
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# ifndef QVERTEXARRAY_H
# define QVERTEXARRAY_H

# include <assert.h>
# include <string.h>

# include <QAxel/QAxelObject.h>
# include <QAxel/QVertex.h>

namespace QAxel {

/*****************************************************************************/
/*! 
  \class QVertexArray QVertexArray.h
  \brief An QVertexArray is used to draw multiple polygons with colors and normals.

  This class implements the normal OpenGL \e vertex \e arrays. They are in
  short a big array of vertexes, which is then drawn with one single command
  using one of the normal OpenGL predefined polygon-types. These types 
  include :
  <ul>
  <li>Points - one vertex per point.
  <li>Lines - separate lines, two vettexes per line.
  <li>LineStrip - draws a line-strip from 0 to 1, 1 to 2, 2 to 3 etc.
  <li>LineLoop - draws a line-strip from 0 to 1, 1 to 2, 2 to 3 and from the
      last back to the first. Makes a 'loop'.
  <li>Triangles - draws separate triangles. 3 vertexes per triangle.
  <li>TriangleStrip - a strip is drawn from the vertexes.
  <li>TriangleFan - all vertexes are drawn as a fan.
  <li>Quads - separate quads are drawn. 4 vertexes per quad.
  <li>QuadStrip - quads are drawn as a strip.
  <li>Polygon - a multi-vertex polygon is drawn from all the vertexes.
  </ul>

  For more information as to how the types work see the <em>OpenGL
  Programming Guide</em>.

  In order to use the \c QVertexArray you must first create an instance of
  this class and make give it the raw vertex-data and the number of
  vertexes in the data. Then you must choose a proper \e type for the
  rendering, see above.  After that you're basically
  done. Additionally you may specify

  The \c QVertexArray does not contain any geometrical, material or other
  transformations, just pure and simple vertexes.

  If the data is passed to \c QVertexArray as a pointer to a pre-created
  datablock, and the data changes, then \c QVertexArray must be
  notified. This is due to caching that probably has copied the data
  from the passed pointer to some other location. If you change your
  data without informing the instance of \c QVertexArray, then the changes
  may or may not show up, depending on your particular implementation
  of OpenGL. Use the method \link invalidateData() \endlink to inform
  \c QVertexArray that the data has changed externally.
*/
/*****************************************************************************/
class QVertexArray : public QAxelObject 
{
    Q_OBJECT

public:

    enum Type { 
	Undefined     = -1, 
	Points        = GL_POINTS, 
	Lines         = GL_LINES,
	LineStrip     = GL_LINE_STRIP,
	LineLoop      = GL_LINE_LOOP,
	Triangles     = GL_TRIANGLES,
	TriangleStrip = GL_TRIANGLE_STRIP,
	TriangleFan   = GL_TRIANGLE_FAN,
	Quads         = GL_QUADS, 
	QuadStrip     = GL_QUAD_STRIP,
	Polygon       = GL_POLYGON 
    } ;
    
    enum NormalMode { 
	PerVertex = 0,
	PerFace   = 1 
    } ;

     QVertexArray(QAxelObject * parent, const Type dataType, const int size) ;
     QVertexArray(QAxelObject * parent, const Type dataType = Undefined) ;
     QVertexArray(QAxelObject * parent, const int size = 0, const Type DataType = Undefined,
	     float * vertices = 0, float * colors = 0, float * normals = 0, const NormalMode Mode = PerFace) ;
    ~QVertexArray(void) ;

    virtual void draw(void) ;
    virtual void drawWithNames(void) ;
  
    void setType(const Type type) ;
    Type type(void) const ;

    void setVertex(const int index, QVertex * vertex) ;
    void setVertex(const int index, const float x, const float y, const float z) ;
    void setVertices(const int size, float * vertices) ;
    void vertex(const int index, float * x, float * y, float * z) const ;
    float * vertices(void) ;

    void setColor(const int index, const QColor & color) ;
    void setColor(const int index, const float red, const float green, const float blue, const float alpha) ;
    void setColors(const int size, float * colors) ;
    const QColor color(const int index) const ;
    float * colors(void) ;

    int size(void) const ;
    void setSize(const int size) ;

    void invalidateData(void) ;

private:

    float * m_Colors ;
    float * m_Vertices ;
    float * m_Normals ;
  
    int m_Size ;
  
    Type m_Type ;
  
    NormalMode m_Mode ;

    bool m_OwnData ;
    bool m_Valid ;
  
    static int m_ArrayCount ;
} ;

} ; // namespace QAxel

# endif // QVERTEXARRAY_H
