/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QCgalPlugin
 * 2006-05-18
 * Julien Wintz - Laurent Rineau
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :         
 ****************************************************************************/

# include <iostream>
# include <list>

# include <QtGui>
# include <QList>
# include <QMap>

# include <QData/QRegion3f.h>
# include <QData/QRegionEdge.h>
# include <QAxel/QMesh.h>
# include <QAxel/QVertex.h>
# include <QAxel/QFace.h>
# include <QCgal/QCgalPlugin.h>

# include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
# include <CGAL/Constrained_Delaunay_triangulation_2.h>
# include <CGAL/Delaunay_mesh_face_base_2.h>
# include <CGAL/Delaunay_mesher_2.h>
# include <CGAL/Delaunay_mesh_criteria_2.h>

using namespace std ;

typedef CGAL::Exact_predicates_inexact_constructions_kernel Kernel;
typedef CGAL::Triangulation_vertex_base_2<Kernel> cdt_vertex_t;
typedef CGAL::Delaunay_mesh_face_base_2<Kernel> cdt_face_t;
typedef CGAL::Triangulation_data_structure_2<cdt_vertex_t, cdt_face_t> cdt_tds_t;
typedef CGAL::Constrained_Delaunay_triangulation_2<Kernel, 
						   cdt_tds_t> constrained_delaunay_triangulation_t;
typedef constrained_delaunay_triangulation_t::Vertex_handle vertex_handle_t;
typedef constrained_delaunay_triangulation_t::Face_handle face_handle_t;
typedef constrained_delaunay_triangulation_t::Finite_vertices_iterator vertex_iterator_t;
typedef constrained_delaunay_triangulation_t::Finite_faces_iterator faces_iterator_t;
typedef constrained_delaunay_triangulation_t::Point vertex_t;
typedef constrained_delaunay_triangulation_t::size_type size_type;
typedef CGAL::Delaunay_mesh_criteria_2<constrained_delaunay_triangulation_t> mesh_criteria_t;

typedef CGAL::Delaunay_mesher_2<constrained_delaunay_triangulation_t, 
				mesh_criteria_t> mesher_t;

QString QCgalPlugin::name() const
{
    return "Cgal" ;
}

QStringList QCgalPlugin::features(void) const
{
    return QStringList() ;
}

QMesh * QCgalPlugin::mesh(QRegion3f * region)
{
    if(region->vertices().size() < 3) return new QMesh ;

    constrained_delaunay_triangulation_t cdt ;
    QMap<QRegionVertex *, vertex_handle_t> map ;
    
    foreach(QRegionVertex * p, region->vertices())
	map[p] = cdt.insert(vertex_t(p->x(), p->y())) ;
    
    foreach(QRegionEdge * e, region->edges())
	cdt.insert_constraint(map[e->source()], map[e->dest()]) ;
    
    if(region->containsHoles()) {

	mesher_t mesher(cdt, mesh_criteria_t());

	std::set<vertex_t> seeds_outside; 
	
	foreach(QRegionEdge * e, region->edges()) {
	    face_handle_t fh;
	    int index;
	    if(cdt.is_edge(map[e->source()], map[e->dest()], fh, index)) {
		// is_edge returns a face handle to the face that
		// is on the right of the edge. As edges are
		// oriented ccw, the right side is outside.
		const vertex_t& pa = fh->vertex(0)->point();
		const vertex_t& pb = fh->vertex(1)->point();
		const vertex_t& pc = fh->vertex(2)->point();
		const vertex_t& o = CGAL::ORIGIN;
		seeds_outside.insert(o + ((pa-o) + (pb-o) + (pc-o)) / 3.);
	    }
	}
	
	mesher.set_seeds(seeds_outside.begin(), 
			 seeds_outside.end(),
			 false); // false means outside
	mesher.refine_mesh();
	
    } else {
	CGAL::refine_Delaunay_mesh_2(cdt, mesh_criteria_t());
    }
    
    QMesh* m = new QMesh();
    
    m->setMeshType(QMesh::Surface) ;
    
    m->setNumberOfVertices(cdt.number_of_vertices()) ;
    
    size_type number_of_faces = 0;
    for(faces_iterator_t fit = cdt.finite_faces_begin(); fit != cdt.finite_faces_end(); ++fit) {
	if(fit->is_in_domain()) ++number_of_faces;
    }
    
    m->setNumberOfFaces(number_of_faces) ;
    
    QMap<vertex_t, int> indexes ;
    
    int v = 0 ;
    for(vertex_iterator_t vit = cdt.finite_vertices_begin(); vit != cdt.finite_vertices_end() ; ++vit) {
	QVertex * vx = new QVertex(v, (float)vit->point().x(), (float)vit->point().y(), 0.0f, m) ;
	m->push_vertex(vx) ;
	indexes[vit->point()] = v++ ;
    }
    
    int f = 0 ;
    for(faces_iterator_t fit = cdt.finite_faces_begin();
	fit != cdt.finite_faces_end();
	++fit)
	if(fit->is_in_domain()) {
	    // if(cdt.locate(cdt.circumcenter(*it)) == )
	    QFace * fc = new QFace(f++, m) ;
	    for(int i = 0 ; i < 3 ; i++) fc->push_index(indexes[fit->vertex(i)->point()]) ;
	    m->push_face(fc) ;
	}
    
    return m ;
}

Q_EXPORT_PLUGIN(QCgalPlugin)
