/*****************************************************************************
 * Axel
 *****************************************************************************
 * QCell
 * 2007-01-10
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 * - Neighborhood informations are no more contained in the cell, since cells
 * are employed together with a Quadtree or an Octree. The latter contains
 * relevant neighborhood informations.
 ****************************************************************************/

# ifndef QCELL_H
# define QCELL_H

# include <QAxel/QBoundingBox.h>

using namespace QAxel ;

namespace QData {

/****************************************************************************/
/*!
  \class QCell QCell.h
  \brief A cell in a subdivision process.

  To apply dynamic programming using cells, they also contain relevant 
  informations as its intersection points with the objects it contains,
  intersections of objects its contains as well as singularities of these
  objects.
*/
/****************************************************************************/
class QCell : public QBoundingBox
{
    Q_OBJECT

public:
    QCell(void) ;
    QCell(QBoundingBox * bb) ;
    QCell(float xmin, float xmax, float ymin=0.0, float ymax=0.0, float zmin=0.0, float zmax=0.0) ;
    
    QList<QPoint3f *> singularPoints(void) { return c_singularities ; }
    void pushSingularPoint(QPoint3f * p) { c_singularities << p ; }
    void setSingularPoints(QList<QPoint3f *> l) { c_singularities = l ; }

    QList<QPoint3f *> xtremPoints(void) { return c_xtremPoints ; }
    void pushXtremPoint(QPoint3f * p) { c_xtremPoints << p ; }
    void setXtremPoints(QList<QPoint3f *> l) { c_xtremPoints = l ; }

    QList<QPoint3f *> ytremPoints(void) { return c_ytremPoints ; }
    void pushYtremPoint(QPoint3f * p) { c_ytremPoints << p ; }
    void setYtremPoints(QList<QPoint3f *> l) { c_ytremPoints = l ; }

//     QList<QPoint3f *> intersectionPoints(void) { return c_intersections ; }
//     void pushIntersectionPoint(QPoint3f * p) { c_intersections << p ; }
//     void setIntersectionPoints(QList<QPoint3f *> l) { c_intersections = l ; }
    
    QList<QPoint3f *> northIntersections(void) { return n_intersections ; }
    QList<QPoint3f *> southIntersections(void) { return s_intersections ; }
    QList<QPoint3f *> eastIntersections(void)  { return e_intersections ; }
    QList<QPoint3f *> westIntersections(void)  { return w_intersections ; }
    
    void pushNorthIntersection(QPoint3f * p) { n_intersections << p ; }
    void pushSouthIntersection(QPoint3f * p) { s_intersections << p ; }
    void pushEastIntersection(QPoint3f * p)  { e_intersections << p ; }
    void pushWestIntersection(QPoint3f * p)  { w_intersections << p ; }
    
    void setNorthIntersections(QList<QPoint3f *> l) { n_intersections = l ; }
    void setSouthIntersections(QList<QPoint3f*> l)  { s_intersections = l ; }
    void setEastIntersections(QList<QPoint3f *> l)  { e_intersections = l ; }
    void setWestIntersections(QList<QPoint3f *> l)  { w_intersections = l ; }
    
    static float abs(float a) ;
    static QCell * intersection(QCell * c1, QCell * c2) ;
    // QCell * normalized  (void) ;
    // QCell * intersect   (QCell * c) ;
    // QCell * intersected (QCell * c) ;
    // QCell * unite       (QCell * c) ;
    // QCell * united      (QCell * c) ;
    
    QList<QCell *> subdivide2D(QPoint3f * center) ;
    QList<QCell *> subdivide3D(QPoint3f * center) ;
    
private:
    QList<QPoint3f *> c_xtremPoints ;
    QList<QPoint3f *> c_ytremPoints ;
    QList<QPoint3f *> c_singularities ; /*! Singularities in the cell */
//    QList<QPoint3f *> c_intersections ; /*! Intersections of objects lying in the cell */
    QList<QPoint3f *> n_intersections ; /*! Intersections of objects lying in the cell with its north border */
    QList<QPoint3f *> s_intersections ; /*! Intersections of objects lying in the cell with its soutth border */
    QList<QPoint3f *> e_intersections ; /*! Intersections of objects lying in the cell with its east border */
    QList<QPoint3f *> w_intersections ; /*! Intersections of objects lying in the cell with its west border */
    
private:
    static int id_ ;
} ;

} ; // namespace QAxel

# endif // QCELL_H
