/******************************************************************************
 * A x e l                                                                    *
 ******************************************************************************
 * QRegionSegmentation                                                        *
 * 2007-04-06                                                                 *
 * Julien Wintz                                                               *
 ******************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis                    *
 ******************************************************************************
 * Comments :                                                                 *
 ******************************************************************************/

# include <QAxel/ObjectManager.h>
# include <QAxel/PluginManager.h>

# include <QData/QRegion3f.h>
# include <QData/QRegionVertex.h>
# include <QData/QRegionEdge.h>
# include <QData/QRegionFace.h>
# include <QData/QRegionSegmentation.h>

# include <QSynaps/QSynapsInterface.h>

# include <QGui/color.h>

using namespace std ;

namespace QData {

int QRegionSegmentation::id_ = 0 ;

QRegionSegmentation::QRegionSegmentation(QRegion3f * region)
{
    this->s_isValid = false ;
    this->s_region  = region ;
    this->s_dlist   = new QDisplayList ;
    
    QString str ;
    setProperty("type", "RegionSegmentation") ;
    setProperty("name", "fseg_"+str.setNum(id_)) ;
    id_++ ;
}

QSegmentationNode_t * QRegionSegmentation::root(void)
{
    return s_root ;
}

bool QRegionSegmentation::isValid(void)
{
    return s_isValid ;
}

/****************************************************************************/
/*! Draws the region segmentation.                                         */
/****************************************************************************/
void QRegionSegmentation::draw(void)
{
    if(!s_dlist->isValid()) {
	s_dlist->begin() ;
	foreach(QRegionEdge * e, s_region->edges())
	    e->box()->draw() ;
	s_dlist->end() ;
    }
    s_dlist->draw() ;
}

/****************************************************************************/
/*! Provided for convenience. */
/****************************************************************************/
void QRegionSegmentation::drawWithNames(void)
{
    ;
}

/****************************************************************************/
/*! Builds the region segmentation.                                         */
/****************************************************************************/
void QRegionSegmentation::build(void) 
{
    Q_ASSERT_X(!s_region->edges().empty(), 
	       "QRegionSegmentation::build", 
	       "The list of edges is empty, nothing to build") ;

    s_region->sort() ;

    QList<QRegionEdge *> s_edges = s_region->edges() ;
    s_tree = alloc(s_edges.size()) ;
    for(int i = 0 ; i < s_edges.size() ; i++)
	new(s_tree+i) QSegmentationNode_t(s_edges[i], s_edges[i]->box()) ;
    s_root = make() ;
    s_isValid = true ;
}

/****************************************************************************/
/*! Allocates memory to compute the segmentation in place.                  */
/****************************************************************************/
QSegmentationNode_t * QRegionSegmentation::alloc(unsigned size)
{
    unsigned i = size ; 
    unsigned acc = 0 ;
    do { acc += i , i = i / 2 + i % 2 ; } while (i > 1) ;
    return new QSegmentationNode_t[acc+1];
}

/****************************************************************************/
/*! Computes the actual segmentation.                                       */
/****************************************************************************/
QSegmentationNode_t * QRegionSegmentation::make(void)
{
    unsigned bsz = s_region->edges().size();
    bool     right = true;
    QSegmentationNode_t * built = s_tree ;
    QSegmentationNode_t * curr  = alloc(bsz/2 + bsz%2) ;
    unsigned i, j ;
    do { 
	if(right) {
	    for(i = 0, j = 0 ; i < bsz-(bsz%2) ; i+=2, j++) 
		new(curr+j) QSegmentationNode_t(built+i, built+i+1, QBoundingBox::unite((built+i)->cell, (built+i+1)->cell)) ;
	    if(bsz%2) *(curr+j++) = *(built+bsz-1) ;
	    built = curr ;
	    curr += j ;
	} else {
	    unsigned nextsz = bsz/2 + bsz % 2 ;
	    built += bsz-1 ;
	    curr  += nextsz-1 ;
	    for(i = 0, j = 0 ; i < bsz-(bsz%2) ; i+= 2, j++) 
		new(curr-j) QSegmentationNode_t(built-i-1, built-i, QBoundingBox::unite((built-i)->cell, (built-i-1)->cell)) ;
	    if(bsz%2) *(curr-j) = *(built-bsz+1) ;
	    built = curr-nextsz+1 ;
	    curr++ ;
	}
	right = !right ;
	bsz = bsz/2+bsz%2 ;
    } while(bsz >1) ;

    return --curr ;
}

/****************************************************************************/
/*! Returns the intersection of the cells contained at the leaves of two
    region segmentations if the segmentations do intersect.
    
    Returns NULL otherwise.

    Remark: No computation is performed.
*/
/****************************************************************************/
QList<QBoundingBox *> QRegionSegmentation::intersect(QRegionSegmentation * r1, QRegionSegmentation * r2)
{
    return intersect(r1->root(), r2->root()) ;
}

QList<QBoundingBox *> QRegionSegmentation::intersect(QSegmentationNode_t * n1, QSegmentationNode_t * n2)
{
    QList<QBoundingBox *> l ;

    if(!QBoundingBox::intersects(n1->cell, n2->cell)) return l ;

    if(n1->isLeaf() && n2->isLeaf())
	l << QBoundingBox::intersect(n1->cell, n2->cell) ;
    else if(n1->isLeaf() && !n2->isLeaf())
	l << intersect(n1, n2->left())
	  << intersect(n1, n2->right()) ;
    else if(n2->isLeaf() && !n1->isLeaf())
	l << intersect(n2, n1->left())
	  << intersect(n2, n1->right()) ;
    else
	l << intersect(n1->left(), n2->left())
	  << intersect(n1->left(), n2->right())
	  << intersect(n1->right(), n2->left())
	  << intersect(n1->right(), n2->right()) ;

    return l ;
}

/****************************************************************************/
/*! */ 
/****************************************************************************/
QList<QSegmentationQuery_t> QRegionSegmentation::query(QRegionSegmentation * r1, QRegionSegmentation * r2)
{
    return query(r1->root(), r2->root()) ;
}

QList<QSegmentationQuery_t> QRegionSegmentation::query(QSegmentationNode_t * n1, QSegmentationNode_t * n2)
{
    QList<QSegmentationQuery_t> l ;

    if(!QBoundingBox::intersects(n1->cell, n2->cell)) return l ;

    if(n1->isLeaf() && n2->isLeaf())
	l << qMakePair(n1, n2) ;
    else if(n1->isLeaf() && !n2->isLeaf())
	l << query(n1, n2->left())
	  << query(n1, n2->right()) ;
    else if(n2->isLeaf() && !n1->isLeaf())
	l << query(n1->left(), n2)
	  << query(n1->right(), n2) ;
    else
	l << query(n1->left(), n2->left())
	  << query(n1->left(), n2->right())
	  << query(n1->right(), n2->left())
	  << query(n1->right(), n2->right()) ;

    return l ;
}

QList<QSegmentationNode_t *> QRegionSegmentation::leaves(void)
{
    QList<QSegmentationNode_t *> lr ;
    QList<QSegmentationNode_t *> lp ;
    
    lp << s_root ;

    while(!lp.empty()) {
	QSegmentationNode_t * n = lp.takeFirst() ;
	if(n->isLeaf()) lr << n ;
	else { lp << n->left() ; lp << n->right() ; }
    }

    return lr ;
}


/****************************************************************************/
/*! Refines a conflict zone defined by two SegmentationNodes.
  
    Subdivides the biggest cell of the \param QSegmentationQuery_t q in two 
    in its biggest direction.

    Returns the new tuple of QSegmentationQuery_t using obtained cells.
 */
/****************************************************************************/
QList<QSegmentationQuery_t> QRegionSegmentation::refine(QSegmentationQuery_t q)
{
    QList<QSegmentationQuery_t> l ;

    QSynapsInterface * iSynaps = dynamic_cast<QSynapsInterface *>(PluginManager::instance()->plugin("Synaps")) ;
    
    QSegmentationNode_t * n1 = q.first ;
    QSegmentationNode_t * n2 = q.second ;

    QRegionEdge * e1 = n1->object() ;
    QRegionEdge * e2 = n2->object() ;
    
    QBoundingBox * b1 = n1->cell ;
    QBoundingBox * b2 = n2->cell ;
    
    float v1 = qMax(b1->width(), b1->height()) ;
    float v2 = qMax(b2->width(), b2->height()) ;

    QBoundingBox * b = (v1 > v2) ? b1 : b2 ;
    QRegionEdge  * e = (v1 > v2) ? e1 : e2 ;

    QList<QPoint3f *> middle ;

    QBoundingBox * bl ;
    QBoundingBox * br ;
    QPoint3f * center ;

    if(b->width() > b->height()) {
	middle = iSynaps->intersectionY(e, b->middleX(), b->ymin(), b1->ymax()) ;
	center = middle.first() ;
	bl = new QBoundingBox(b->xmin(), center->x(), b->ymin(), b->ymax()) ;
	br = new QBoundingBox(center->x(), b->xmax(), b->ymin(), b->ymax()) ;
    } else { 
	middle = iSynaps->intersectionX(e, b->middleY(), b->xmin(), b1->xmax()) ;
	center = middle.first() ;
	bl = new QBoundingBox(b->xmin(), b->xmax(), b->ymin(), center->y()) ;
	br = new QBoundingBox(b->xmin(), b->xmax(), center->y(), b->ymax()) ;
    }

    b->setColor(Qt::red) ;
    center->setSize(1e-3) ;
    ObjectManager::instance()->addObject(center) ;

    QSegmentationNode_t * nl = new QSegmentationNode_t(e, bl) ;
    QSegmentationNode_t * nr = new QSegmentationNode_t(e, br) ;

    if(v1 > v2) {
 	if(QBoundingBox::intersects(bl,b2)) l << qMakePair(nl, n2) ;
 	if(QBoundingBox::intersects(br,b2)) l << qMakePair(nr, n2) ;
    } else {
 	if(QBoundingBox::intersects(b1,bl)) l << qMakePair(n1, nl) ;
 	if(QBoundingBox::intersects(b1,br)) l << qMakePair(n1, nr) ;
    }

    if(v1 > v2) {
 	if(QBoundingBox::intersects(bl,b2)) { ObjectManager::instance()->addObject(bl) ; ObjectManager::instance()->addObject(b2) ; }
 	if(QBoundingBox::intersects(br,b2)) { ObjectManager::instance()->addObject(br) ; ObjectManager::instance()->addObject(b2) ; }
    } else {
 	if(QBoundingBox::intersects(b1,bl)) { ObjectManager::instance()->addObject(b1) ; ObjectManager::instance()->addObject(bl) ; }
 	if(QBoundingBox::intersects(b1,br)) { ObjectManager::instance()->addObject(b1) ; ObjectManager::instance()->addObject(br) ; }
    }

    cout << FG_GREEN << __func__ << " : retuning " << l.size() << " queries" << NOCOLOR << endl ; 

    return l ;
}

} ; // namespace QData
