/*****************************************************************************
 * A x e l
 *****************************************************************************
 * Commands.h
 * 2006-03-31
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 * The set of Axel's commands made available in the undo/redo framework
 ****************************************************************************/

# ifndef COMMANDS_H
# define COMMANDS_H

# include <QUndoCommand>

# include <QAxel/QBoundingBox.h>
# include <QAxel/QPoint3f.h>
# include <QAxel/QPlane.h>
# include <QAxel/QRationalCurve.h>
# include <QAxel/QBSplineCurve.h>
# include <QAxel/QNurbsCurve.h>
# include <QAxel/QImplicitCurve2D.h>
# include <QAxel/QImplicitCurve3D.h>
# include <QAxel/QImplicitCurve4D.h>
# include <QAxel/QRationalSurface.h>
# include <QAxel/QBSplineSurface.h>
# include <QAxel/QNurbsSurface.h>
# include <QAxel/QImplicitSurface.h>

# include <QSisl/QSislInterface.h>

# include <QAxel/PluginManager.h>

# include <QGui/Viewer.h>

using namespace QAxel ;

class CreatePointCommand : public QUndoCommand
{	
public:
	CreatePointCommand(Viewer * v, float x, float y, float z) {
		this->v = v ;
		p = new QPoint3f(x,y,z) ;
		setText("Create point") ;
	}
	~CreatePointCommand(void) {
		delete p ;
	}
	
	void setName(QString name) { p->setName(name) ; }
	void setColor(int r, int g, int b) { p->setRGB(r,g,b) ; }
	
	virtual void undo(void) { v->removeObject(p) ; }
	virtual void redo(void) { v->addObject(p) ; }

private:
	Viewer * v ;
	QPoint3f * p ;
} ;

class CreatePlaneCommand : public QUndoCommand
{	
public:
	CreatePlaneCommand(Viewer * v, float a, float b, float c, float d) {
		this->v = v ;
		p = new QPlane(a,b,c,d,1.0) ;
		setText("Create plane") ;
	}
	~CreatePlaneCommand(void) {
		delete p ;
	}
	
	virtual void undo(void) { v->removeObject(p) ; }
	virtual void redo(void) { v->addObject(p) ; }

private:
	Viewer * v ;
	QPlane * p ;
} ;

class CreateImplicitCurve2DCommand : public QUndoCommand
{
public :
	CreateImplicitCurve2DCommand(Viewer * v, QString polynomial, QBoundingBox * bb, QImplicitCurve::Method method) {
		this->v = v ;
		c = new QImplicitCurve2D(polynomial, bb, method) ;
		setText("Create implicit planar curve") ;
	}
	~CreateImplicitCurve2DCommand(void) {
		delete c ;
	}
	
	virtual void undo(void) { v->removeObject(c) ; }
	virtual void redo(void) { v->addObject(c) ; }

private:
	Viewer * v ;
	QImplicitCurve2D * c ;	
} ;

class CreateImplicitCurve3DCommand : public QUndoCommand
{
public :
	CreateImplicitCurve3DCommand(Viewer * v, QString polynomial1, QString polynomial2, QBoundingBox * bb, QImplicitCurve::Method method) {
		this->v = v ;
		c = new QImplicitCurve3D(polynomial1, polynomial2, bb, method) ;
		setText("Create implicit spatial curve") ;
	}
	~CreateImplicitCurve3DCommand(void) {
		delete c ;
	}
	
	virtual void undo(void) { v->removeObject(c) ; }
	virtual void redo(void) { v->addObject(c) ; }

private:
	Viewer * v ;
	QImplicitCurve3D * c ;	
} ;

class CreateRationalCurveCommand : public QUndoCommand
{
  public :
    CreateRationalCurveCommand(Viewer * v, QString px, QString py, QString pz, QString pw, float tmin, float tmax) {
      this->v = v ;
      c = new QRationalCurve(px, py, pz, pw, tmin, tmax) ;
      setText("Create rational curve") ;
    }
    ~CreateRationalCurveCommand(void) {
      delete c ;
    }
    
    virtual void undo(void) { v->removeObject(c) ; }
    virtual void redo(void) { v->addObject(c) ; }

  private:
    Viewer * v ;
    QRationalCurve * c ;  
} ;

class CreateBSplineCurveCommand : public QUndoCommand
{
public :
    CreateBSplineCurveCommand(Viewer * viewer ,int iopen, int ikind, int order, int dimension) {
	v = viewer ;
	c = new QBSplineCurve ;
	
	QList<QPoint3f *> l ;
	QPoint3f * pt1 = new QPoint3f(-1.0, 0.0, 0.0) ; l << pt1 ;
	QPoint3f * pt2 = new QPoint3f(-0.5, 0.5, 0.0) ; l << pt2 ;
	QPoint3f * pt3 = new QPoint3f(0.5, 0.5, 0.0) ;  l << pt3 ;
	QPoint3f * pt4 = new QPoint3f(1.0, 0.0, 0.0) ;  l << pt4 ;
	
	QSislInterface * iSisl = dynamic_cast<QSislInterface *>(PluginManager::instance()->plugin("Sisl")) ;
	if(iSisl) {
	    c = iSisl->approximateBSplineCurve(l, iopen, ikind, order, dimension) ;
	    c->compute() ;
	} else {
	    QMessageBox::information(0, "Axel","Approximation : Sisl plugin not loaded","&Ok",0) ;
	}

	c->setMode(QAxelObject::Edit) ;
//	v->addObjectToSelection(c) ;

	setText("Create bspline curve") ;
    }
    
    ~CreateBSplineCurveCommand(void) {
	delete c ;
    }
    
    virtual void undo(void) { v->removeObject(c) ; }
    virtual void redo(void) { v->addObject(c) ;    }
    
private:
    Viewer * v ;
    QBSplineCurve * c ;  
} ;

class CreateBSplineSurfaceCommand : public QUndoCommand
{
public :
    CreateBSplineSurfaceCommand(Viewer * viewer ,int order1, int order2, int number1, int number2, int dimension) {
	v = viewer ;

	s = new QBSplineSurface(NULL) ;
	s->order1 = order1 ;
	s->order2 = order2 ;
	s->number1 = number1 ;
	s->number2 = number2 ;
	s->idim = dimension ;
	
	for(int i = 0 ; i < number2 ; i++ )
	    for(int j = 0 ; j < number1 ; j++)
		s->push_point(new QPoint3f(j,0,i)) ;

	s->computeKnots1() ;
	s->computeKnots2() ;
	
 	foreach(QPoint3f * p, s->points)
 	    p->setBlackListed(true) ;
	
	s->compute() ;
	s->setMode(QAxelObject::Edit) ;
	v->addObject(s) ;

	s->compute() ;

	setText("Create bspline surface") ;
//	cout << __func__ << " : created bspline surface : " << endl ;
	
    }
    
   ~CreateBSplineSurfaceCommand(void) {
	delete s ;
    }
    
    virtual void undo(void) { v->removeObject(s) ; }
    virtual void redo(void) { v->addObject(s) ;    }
    
private:
    Viewer * v ;
    QBSplineSurface * s ;  
} ;

class CreateRationalSurfaceCommand : public QUndoCommand
{
  public :
    CreateRationalSurfaceCommand(Viewer * v, QString px, QString py, QString pz, QString pw, float smin, float smax, float tmin, float tmax) {
      this->v = v ;
      s = new QRationalSurface(px, py, pz, pw, smin, smax, tmin, tmax) ;
      setText("Create rational surface") ;
    }
    ~CreateRationalSurfaceCommand(void) {
      delete s ;
    }
    
    virtual void undo(void) { v->removeObject(s) ; }
    virtual void redo(void) { v->addObject(s) ; }

  private:
    Viewer * v ;
    QRationalSurface * s ;  
} ;

class CreateImplicitSurfaceCommand : public QUndoCommand
{
  public :
    CreateImplicitSurfaceCommand(Viewer * v, QString eq, QBoundingBox * bb) {
	this->v = v ;
	s = new QImplicitSurface(eq, bb) ;
	setText("Create implicit surface") ;
    }

    ~CreateImplicitSurfaceCommand(void) {
	delete s ;
    }
    
    virtual void undo(void) { v->removeObject(s) ; }
    virtual void redo(void) { v->addObject(s) ; }

  private:
    Viewer * v ;
    QImplicitSurface * s ;  
} ;


# endif
