/****************************************************************************
 * A x e l
 *****************************************************************************
 * ObjectPropertiesDock
 * 2006-03-31
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# include <iostream>

# include <QSplitter>

# include <QGui/Viewer.h>
# include <QGui/ObjectPropertiesDock.h>
# include <QGui/PropertyEditorModel.h>
# include <QGui/QPropertyEditor.h>
# include <QGui/QParameterSpaceView.h>

# include <QAxel/QPoint3f.h>
# include <QAxel/QPlane.h>
# include <QAxel/QConstraintDistance.h>
# include <QAxel/QCurve.h>
# include <QAxel/QRationalSurface.h>
# include <QAxel/QBSplineSurface.h>
# include <QAxel/QNurbsSurface.h>
# include <QAxel/QImplicitSurface.h>
# include <QAxel/QVertex.h>
# include <QAxel/QEdge.h>
# include <QAxel/QFace.h>
# include <QAxel/QMesh.h>

using namespace QAxel ;

ObjectPropertiesDock::ObjectPropertiesDock(Viewer * viewer, const QString & title, QWidget * parent) : QDockWidget(title, parent), m_properties(0) 
{
    dirty = false ;

    splitter = new QSplitter ;
    splitter->setOrientation(Qt::Vertical) ;
    
    viewer_ = viewer ;
    m_editor = new QPropertyEditor(this) ;

    connect(m_editor, SIGNAL(propertyChanged(IProperty*)), this, SLOT(firePropertyChanged(IProperty*)));
    connect(m_editor->editorModel(), SIGNAL(resetProperty(QString)), this, SLOT(resetProperty(QString)));
    
    splitter->addWidget(m_editor) ;
    
    setWidget(splitter) ;
}


bool ObjectPropertiesDock::isReadOnly() const
{
    return m_editor->isReadOnly();
}

void ObjectPropertiesDock::setReadOnly(bool readOnly)
{
    m_editor->setReadOnly(readOnly);
}

Viewer * ObjectPropertiesDock::viewer(void) const
{
    return viewer_ ;
}

IProperty *ObjectPropertiesDock::propertyByName(IProperty *p, const QString &name)
{
    if (p->propertyName() == name)
	return p;
    
    if (p->kind() == IProperty::Property_Group) {
	IPropertyGroup *g = static_cast<IPropertyGroup*>(p);
	for (int i=0; i<g->propertyCount(); ++i)
	    if (IProperty *c = propertyByName(g->propertyAt(i), name))
		return c;
    }
    
    return 0;
}

void ObjectPropertiesDock::setPropertyValue(const QString &name, const QVariant &value, bool changed)
{
    if (isReadOnly())
	return;

    if (IProperty *p = propertyByName(m_editor->initialInput(), name)) {
	if (p->value() != value)
	    p->setValue(value);
	
	p->setChanged(changed);
	p->setDirty(false);
	
	m_editor->editorModel()->refresh(p);
    }
}

void ObjectPropertiesDock::firePropertyChanged(IProperty *p)
{
    if (isReadOnly()) return;
    
    if (object() && p->parent() && p->propertyName() == "Name") {
	QAxelObject * o = object() ;
	QString s = qvariant_cast<QString>(p->value()) ;
	o->setName(s) ;
    }
	
    if (object() && p->parent() && p->propertyName() == "Color") {
	QAxelObject * o = object() ;
	QColor c = qvariant_cast<QColor>(p->value()) ;
	o->setRGB(c.red(), c.green(), c.blue()) ;
    }
	
    if (object() && p->parent() && p->propertyName() == "Draw Bounding Box") {
	if(QCurve * o = dynamic_cast<QCurve *>(object())) {
	    bool b = qvariant_cast<bool>(p->value()) ;
	    o->setBBDrawn(b) ;
	} else if(QSurface * o = dynamic_cast<QSurface *>(object())) {
	    bool b = qvariant_cast<bool>(p->value()) ;
	    o->setBBDrawn(b) ;		
	}
    }
    
    if (object() && p->parent() && p->propertyName() == "Draw Singularities") {
	if(QImplicitSurface * o = dynamic_cast<QImplicitSurface *>(object())) {
	    bool b = qvariant_cast<bool>(p->value()) ;
	    o->setSingularitiesDrawn(b) ;		
	}
    }
    
    if (object() && p->parent() && p->propertyName() == "Draw Edges") {
	if(QMesh * o = dynamic_cast<QMesh *>(object())) {
	    bool b = qvariant_cast<bool>(p->value()) ;
	    o->drawEdges = b ;		
	}
    }
	
    if (object() && p->parent() && p->propertyName() == "x") {
	QPoint3f * point = qobject_cast<QPoint3f *>(object());
	if (point) {
	    double value = qvariant_cast<double>(p->value()) ;
	    point->setx(value);
	}
    }
    
    if (object() && p->parent() && p->propertyName() == "y") {
	QPoint3f * point = qobject_cast<QPoint3f *>(object());
	if (point) {
	    double value = qvariant_cast<double>(p->value()) ;
	    point->sety(value);
	}
    }

    if (object() && p->parent() && p->propertyName() == "z") {
	QPoint3f * point = qobject_cast<QPoint3f *>(object());
	if (point) {
	    double value = qvariant_cast<double>(p->value()) ;
	    point->setz(value);
	}
    }

    if (object() && p->parent() && p->propertyName() == "a") {
	QPlane * plane = qobject_cast<QPlane *>(object());
	if (plane) {
	    double value = qvariant_cast<double>(p->value()) ;
	    plane->seta(value);
	}
    }
	
    if (object() && p->parent() && p->propertyName() == "b") {
	QPlane * plane = qobject_cast<QPlane *>(object());
	if (plane) {
	    double value = qvariant_cast<double>(p->value()) ;
	    plane->setb(value);
	}
    }
    
    if (object() && p->parent() && p->propertyName() == "c") {
	QPlane * plane = qobject_cast<QPlane *>(object());
	if (plane) {
	    double value = qvariant_cast<double>(p->value()) ;
	    plane->setc(value);
	}
    }
    
    if (object() && p->parent() && p->propertyName() == "d") {
	QPlane * plane = qobject_cast<QPlane *>(object());
	if (plane) {
	    double value = qvariant_cast<double>(p->value()) ;
	    plane->setd(value);
	}
    }
    
    if (object() && p->parent() && p->propertyName() == "size") {
	QPlane * plane = qobject_cast<QPlane *>(object());
	if (plane) {
	    double value = qvariant_cast<double>(p->value()) ;
	    plane->setSize(value);
	}
    }
    
    if (object() && p->parent() && p->propertyName() == "value") {
	QConstraintDistance * distance = qobject_cast<QConstraintDistance *>(object());
	if (distance) {
	    double value = qvariant_cast<double>(p->value()) ;
	    distance->setValue(value);
	}
    }

    viewer_->updateGL() ;
 
    emit propertyChanged() ;
}

void ObjectPropertiesDock::clearDirty(IProperty *p)
{
    p->setDirty(false);
    
    if (p->kind() == IProperty::Property_Normal)
	return;
    
    IPropertyGroup *g = static_cast<IPropertyGroup*>(p);
    for (int i=0; i<g->propertyCount(); ++i)
	clearDirty(g->propertyAt(i));
}

void ObjectPropertiesDock::setObject(QAxelObject *object)
{
//    if(object->inherits("QParametricCurve") || object->inherits("QParametricSurface")) {
    if((object->type() == "BSplineCurve" 
	|| object->type() == "BSplineSurface"
	|| object->type() == "RationalCurve"
	|| object->type() == "RationalSurface")
       && !dirty) {
	parameterSpaceView = new QParameterSpaceView(object, this) ;
	dirty = !dirty ;
	connect(parameterSpaceView, SIGNAL(evaluation(QAxelObject *)), this, SIGNAL(addObject(QAxelObject *))) ;
	connect(parameterSpaceView, SIGNAL(updated()), this, SIGNAL(updated())) ;
	splitter->addWidget(parameterSpaceView) ;
    }

    //if(object->type() == "BSplineCurve") return ;
    
    if (m_editor->initialInput())
	clearDirty(m_editor->initialInput());
    
    m_object = object;

    IPropertyGroup *old_properties = m_properties;
    m_properties = 0;

    if (m_object) {
	PropertyCollection * collection = new PropertyCollection(QLatin1String("<root>"));
	createPropertySheet(collection, object);
	m_properties = collection;
    }
    
    m_editor->setInitialInput(m_properties);
    
    delete old_properties;
}

void ObjectPropertiesDock::clear(void) 
{
    PropertyCollection * collection = new PropertyCollection(QLatin1String("<root>")) ;
    m_properties = collection;
    m_editor->setInitialInput(m_properties);

    if(dirty) {
	delete parameterSpaceView ; // This should remove it from the splitter :-|
	dirty = !dirty ;
    }
}

void ObjectPropertiesDock::resetProperty(const QString &)
{

}

QString ObjectPropertiesDock::currentPropertyName() const
{
    QModelIndex index = m_editor->selectionModel()->currentIndex();
    if (index.isValid()) {
	IProperty *property = static_cast<IProperty*>(index.internalPointer());
	
	while (property && property->isFake())
	    property = property->parent();
	
	if (property)
	    return property->propertyName();
    }
    
    return QString() ;
}
