/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QGraphWidget
 * 2006-07-20
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# include <QDebug>
# include <QColor>
# include <QGraphicsScene>
# include <QWheelEvent>

# include <QAxel/QBoundingBox.h>
# include <QData/QRegion3f.h>
# include <QData/QRegionSegmentation.h>
# include <QGui/Viewer.h>
# include <QGui/QGraphWidget.h>

# include <iostream>
# include <math.h>

using namespace std ;
using namespace QAxel ;
using namespace QData ;

QGraphWidget::QGraphWidget(QWidget * parent) : QGraphicsView(parent)
{
    QGraphicsScene *scene = new QGraphicsScene(this);
    scene->setItemIndexMethod(QGraphicsScene::NoIndex);
    scene->setSceneRect(-5000, -5000, 5000, 5000);
    setScene(scene);

    setCacheMode(CacheBackground);
    setRenderHint(QPainter::Antialiasing);
    setTransformationAnchor(AnchorUnderMouse);
    setResizeAnchor(AnchorViewCenter);

    state = NONE ;
}

void QGraphWidget::setGraph(QGraph * g)
{
    this->graph = g ;
    update() ;
}

void QGraphWidget::update(void)
{
    clear() ;
    foreach(QGraphNode * n, graph->nodes()) nodeMap.insert(n, new QGraphicNode(this, n->label(), n->lightColor(), n->darkColor())) ;
    foreach(QGraphEdge * e, graph->edges()) edgeMap.insert(e, new QGraphicEdge(nodeMap.value(e->sourceNode()), nodeMap.value(e->destNode()))) ;
    foreach(QGraphicNode * n, nodeMap.values()) scene()->addItem(n) ;	
    foreach(QGraphicEdge * e, edgeMap.values()) scene()->addItem(e) ;
    setDirectedTreeLayout() ;
}

void QGraphWidget::clear(void) 
{
    foreach(QGraphicsItem * i, scene()->items()) scene()->removeItem(i) ;
    edgeMap.clear() ; 
    nodeMap.clear() ;	
}

void QGraphWidget::keyPressEvent(QKeyEvent *event)
{
    switch (event->key()) {
    case Qt::Key_Up:
        break;
    case Qt::Key_Down:
        break;
    case Qt::Key_Left:
        break;
    case Qt::Key_Right:
        break;
    case Qt::Key_Plus:
        scaleView(1.2);
        break;
    case Qt::Key_Minus:
        scaleView(1 / 1.2);
        break;
    case Qt::Key_Space:
    case Qt::Key_Enter:
    default:
        QGraphicsView::keyPressEvent(event);
    }
}

void QGraphWidget::mousePressEvent(QMouseEvent * e) 
{
    // Navigation must not occur if a node is under the mouse
    QPointF p = mapToScene(e->pos());
    QList<QGraphicsItem*> items = scene()->items(p);

    // Callback visualization of objects contained in the nodes
    foreach(QGraphicsItem * item, items) {
	
	if(dynamic_cast<QGraphicNode *>(item)) {

	if(graph->type() == QGraph::AugmentedInfluenceGraph) {
	    QGraphNode * n = nodeMap.key(dynamic_cast<QGraphicNode *>(item)) ;
	    Q_ASSERT(n->n_type == QGraphNode::Graph) ;
	    QRegion3f * r = n->region() ;
	    if(r)
		if(e->button() == Qt::LeftButton)
		    viewer->addObject(r) ;
		else if(e->button() == Qt::RightButton)
		    viewer->addObject(r->segmentation()) ;
	} 
	
	if(graph->type() == QGraph::Octree) {
	    if(QBoundingBox * c = (QBoundingBox *)(nodeMap.keys(dynamic_cast<QGraphicNode *>(item)).first()->cell()))
		if(e->button() == Qt::LeftButton)
		    viewer->addObject(c) ;
	}

	}
    }
	
    if(items.size() == 0) {
	if(e->button() == Qt::RightButton) {
	    start = e->pos();
	    viewport()->setCursor(QCursor(Qt::SizeAllCursor)) ;
	    setState(PANNING);
	}
    } else QGraphicsView::mousePressEvent(e);
}

void QGraphWidget::mouseReleaseEvent(QMouseEvent * e)
{

    // Navigation must not occur if a node is under the mouse
    QPointF p = mapToScene(e->pos());
    QList<QGraphicsItem*> items = scene()->items(p);

    // Callback visualization of objects contained in the nodes
    foreach(QGraphicsItem * item, items) {

	if(dynamic_cast<QGraphicNode *>(item)) {

	if(graph->type() == QGraph::AugmentedInfluenceGraph)
	    if(QRegion3f * r = nodeMap.key(dynamic_cast<QGraphicNode *>(item))->region())
		if(e->button() == Qt::LeftButton)
		    viewer->removeObject(r) ;
		else if(e->button() == Qt::RightButton)
		    viewer->removeObject(r->segmentation()) ;
	
	if(graph->type() == QGraph::Octree)
	    if(QBoundingBox * c = (QBoundingBox *)(nodeMap.keys(dynamic_cast<QGraphicNode *>(item)).first()->cell()))
		if(e->button() == Qt::LeftButton)
		    viewer->removeObject(c) ;

	}
    }


    viewport()->setCursor(QCursor(Qt::ArrowCursor));

    if(getState() == PANNING) {
	setState(NONE) ;
    } else QGraphicsView::mouseReleaseEvent(e) ;
}

void QGraphWidget::mouseMoveEvent(QMouseEvent * e)
{
    if(getState() == PANNING) {
	QPointF p = e->pos() ;  

	double accel = 1.0;
	double dx = (start.x() - p.x()) * accel ;
	double dy = (start.y() - p.y()) * accel ;
	translate(dx,dy) ;

	start = p;
    } else QGraphicsView::mouseMoveEvent(e) ;
}

void QGraphWidget::wheelEvent(QWheelEvent *event)
{
    scaleView(pow((double)2, -event->delta() / 240.0));
}

void QGraphWidget::drawForeground(QPainter * p, const QRectF & rect)
{
    Q_UNUSED(p) ;
    Q_UNUSED(rect) ;

//     p->save();

//     p->setPen(Qt::black);
//     QColor color(Qt::lightGray) ;
//     color.setAlpha(127) ;
//     p->setBrush(QBrush(color));

//     QRectF b(rect) ;
//     b.setHeight(rect.height()/10) ;
//     p->drawRect(b);

//     QFontMetrics metrics = QFontMetrics(font());
//     int border = qMax(4, metrics.leading());

//     QString text = "Graph visualization" ;

//     QRect fontRect = metrics.boundingRect(0, 0, b.width() - 2*border, int(b.height()*0.125), Qt::AlignCenter | Qt::TextWordWrap, text);

//     p->setRenderHint(QPainter::TextAntialiasing);
//     p->setPen(Qt::darkGray);
//     p->drawText((b.width() - fontRect.width())/2, border,
//                      fontRect.width(), fontRect.height(),
//                      Qt::AlignCenter | Qt::TextWordWrap, text);

//     p->restore();

}

void QGraphWidget::drawBackground(QPainter *painter, const QRectF &rect)
{
    Q_UNUSED(rect) ;
    Q_UNUSED(painter) ;
}

void QGraphWidget::scaleView(qreal scaleFactor)
{
    scale(scaleFactor, scaleFactor);
}


void QGraphWidget::resetNodesPositions(void)
{
    foreach(QGraphicNode * n, nodeMap.values()) n->setPos(-1.0f, -1.0f) ;
}

void QGraphWidget::setRandomLayout(void)
{
    foreach(QGraphicEdge * e, edgeMap.values()) scene()->removeItem((QGraphicsItem*)e) ;
    edgeMap.clear() ; 
    foreach(QGraphicNode * n, nodeMap.values()) {
        float w = (float)((rand()%100) / 100.0 * scene()->width()) ;
        float h = (float)((rand()%100) / 100.0 * scene()->height()) ;
        n->setPos(w, h) ;
    }
    foreach(QGraphEdge * e, graph->edges()) edgeMap.insert(e, new QGraphicEdge(nodeMap.value(e->sourceNode()), nodeMap.value(e->destNode()))) ;	
    foreach(QGraphicEdge * e, edgeMap.values()) scene()->addItem(e) ;
}

void QGraphWidget::setConcentricLayout(void) 
{
    foreach(QGraphicEdge * e, edgeMap.values()) scene()->removeItem((QGraphicsItem*)e) ; 
    edgeMap.clear() ; 
	
    QPointF center = sceneRect().center( );
    double azimutDelta = 15. ;
    double circleInterval = 10. ;
    int	nodesPerCircle = (int)(360/azimutDelta) ;
	
    int i = 0 ;
    foreach(QGraphicNode * n, nodeMap.values())  {
	double azimutIndex = (i % nodesPerCircle);
	double azimut = azimutIndex * azimutDelta;

	int circleIndex = 1 + ( i / nodesPerCircle );
	double cx = sin( azimut * 3.14156 / 180. ) * (circleIndex * circleInterval);
	double cy = cos( azimut * 3.14156 / 180. ) * (circleIndex * circleInterval);

	n->setPos(center.x()+cx, center.y()+cy) ;
	i++ ;
    }
	
    foreach(QGraphEdge * e, graph->edges()) edgeMap.insert(e, new QGraphicEdge(nodeMap.value(e->sourceNode()), nodeMap.value(e->destNode()))) ;	
    foreach(QGraphicEdge * e, edgeMap.values()) scene()->addItem(e) ;	
}

void QGraphWidget::setColimaconLayout(void)
{
    foreach(QGraphicEdge * e, edgeMap.values()) scene()->removeItem((QGraphicsItem*)e) ; 
    edgeMap.clear() ;
	
    QPointF center = sceneRect().center( );
    double azimutDelta = 15. ;
    double circleInterval = 10. ;

    int i = 0 ;
    foreach(QGraphicNode * n, nodeMap.values())  {
	double azimut = i * azimutDelta;
	double cx = sin(azimut * 3.14156 / 180.) * (log( 1. + i ) * 10 * circleInterval) ;
	double cy = cos(azimut * 3.14156 / 180.) * (log( 1. + i ) * 10 * circleInterval) ;

	n->setPos(center.x()+cx, center.y()+cy) ;
	i++ ;
    }

    foreach(QGraphEdge * e, graph->edges()) edgeMap.insert(e, new QGraphicEdge(nodeMap.value(e->sourceNode()), nodeMap.value(e->destNode()))) ;	
    foreach(QGraphicEdge * e, edgeMap.values()) scene()->addItem(e) ;
}

void QGraphWidget::setDirectedTreeLayout() 
{
    foreach(QGraphicEdge * e, edgeMap.values()) scene()->removeItem((QGraphicsItem*)e) ; 
    edgeMap.clear() ;
	
    // cout << "---------------------------" << endl ;
    // cout << "setDirectedTreeLayout(void)" << endl ;
    // cout << "---------------------------" << endl ;
	
    marked.clear() ;
    resetNodesPositions() ;
	
    // Configure tree bounding box
    QPointF bbox[2] ;
    bbox[0].setX(scene()->sceneRect().x()) ;
    bbox[0].setY(scene()->sceneRect().y()) ;
    bbox[1].setX(scene()->sceneRect().x()+scene()->sceneRect().width()) ;
    bbox[1].setY(scene()->sceneRect().y()+scene()->sceneRect().height()) ;
	
    QPointF point(	bbox[0].x() + ((bbox[1].x()-bbox[0].x())/2), 
			bbox[0].y() + ((bbox[1].y()-bbox[0].y())/2)) ;

    int depth = 0 ;
    int delta = 60 ;
    int siblings = 0 ;

    // Layout all graph subgraph
    int i = 0 ;
    foreach(QGraphNode * n, graph->rootNodes())
    {
	// cout << "A Root Node: " << n->label().toLatin1().data() << endl ;
	point.setX(point.x() + i*delta) ;
	setDirectedTreeLayout(n, point, depth, delta, siblings);
	i++ ;
    }
	
    foreach(QGraphEdge * e, graph->edges()) edgeMap.insert(e, new QGraphicEdge(nodeMap.value(e->sourceNode()), nodeMap.value(e->destNode()))) ;
    foreach(QGraphicEdge * e, edgeMap.values()) scene()->addItem(e) ;
}

void QGraphWidget::setDirectedTreeLayout(QGraphNode * n, QPointF& point, int depth, int delta, int siblings) 
{
    Q_UNUSED(siblings) ;
    // cout << "---------------------------" << endl ;
    // cout << "setDirectedTreeLayout(" << n->label().toLatin1().data() << ", ("
    // 	 << point.x() << "," << point.y() << "), " << depth << "," << siblings << ")" << endl ;
    // cout << "---------------------------" << endl ;
	
    if (marked.find(n) != marked.end()) return ;
    else marked.insert(n) ;

    int laidOut = 0 ;
    foreach(QGraphNode * node, n->outNodesList()) {
	QGraphicNode * gNode = nodeMap.value(node) ;

	if ( gNode->pos().x() < 0.f && 
	     gNode->pos().y() < 0.f  )
	{
	    if ( laidOut > 0 ) 
		point.setX(point.x() + delta) ;

	    setDirectedTreeLayout( node, point, depth+1, delta, n->outNodesList().size());
	    laidOut++;
	}
    }

    float x ;
    if(n->isLeaf()) { x = point.x() ; /* cout << n->label().toLatin1().data() << " is a leaf" << endl ; */ }
    else 			x = middleAbsissaOfChildNodes(n) ;
    float y = point.y() + ((float)delta*(float)depth) ;
    // cout << "Setting " << n->label().toLatin1().data() << " coordinates : (" << x << "," << y << ")" << endl ;
    nodeMap.value(n)->setPos(x, y) ;
} 

float QGraphWidget::middleAbsissaOfChildNodes(QGraphNode * n)
{
    // cout << "looking child nodes of " << n->label().toLatin1().data() << endl ;
    float max = -99999999.0 ;
    float min = +99999999.0 ;
    foreach(QGraphNode * it, n->outNodesList()) {
	// cout << "- examining " << it->label().toLatin1().data() << endl ;
	if(nodeMap.value(it)->pos().x() < min) min = nodeMap.value(it)->pos().x() ;
	if(nodeMap.value(it)->pos().x() > max) max = nodeMap.value(it)->pos().x() ;
    }
    return (max+min)/2 ;
}
