/****************************************************************************
**
** Copyright (C) 2006-2006 Trolltech ASA. All rights reserved.
**
** This file is part of the example classes of the Qt Toolkit.
**
** Licensees holding valid Qt Preview licenses may use this file in
** accordance with the Qt Preview License Agreement provided with the
** Software.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
** information about Qt Commercial License Agreements.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

# include <QColor>
# include <QGraphicsScene>
# include <QGraphicsSceneMouseEvent>
# include <QPainter>
# include <QStyleOption>

# include <QGui/QGraphicEdge.h>
# include <QGui/QGraphicNode.h>
# include <QGui/QGraphWidget.h>


QGraphicNode::QGraphicNode(QGraphWidget *graphWidget, QString label, QColor lightColor, QColor darkColor) : 
	graph(graphWidget), 
	label(label),
	n_lightColor(lightColor), 
	n_darkColor(darkColor)
{
    setFlag(ItemIsMovable);
    setZValue(1);	
}

void QGraphicNode::setColors(QColor lightColor, QColor darkColor) 
{
    this->n_lightColor = lightColor ;
    this->n_darkColor = darkColor ;
}

void QGraphicNode::setLightColor(QColor lightColor)
{
    this->n_lightColor = lightColor ;
}

void QGraphicNode::setDarkColor(QColor darkColor) 
{
    this->n_darkColor = darkColor ;
}

void QGraphicNode::addEdge(QGraphicEdge *edge)
{
    edgeList << edge;
    edge->adjust();
}

QList<QGraphicEdge *> QGraphicNode::edges() const
{
    return edgeList;
}

void QGraphicNode::calculateForces()
{
    if (!scene() || scene()->mouseGrabberItem() == this) {
        newPos = pos();
        return;
    }
    
    // Sum up all forces pushing this item away
    qreal xvel = 0;
    qreal yvel = 0;
    foreach (QGraphicsItem *item, scene()->items()) {
        QGraphicNode *node = qgraphicsitem_cast<QGraphicNode *>(item);
        if (!node)
            continue;

        QLineF line(mapFromItem(node, 0, 0), QPointF(0, 0));
        qreal dx = line.dx();
        qreal dy = line.dy();
        double l = 2.0 * (dx * dx + dy * dy);
        if (l > 0) {
            xvel += (dx * 150.0) / l;
            yvel += (dy * 150.0) / l;
        }
    }

    // Now subtract all forces pulling items together
    double weight = (edgeList.size() + 1) * 10;
    foreach (QGraphicEdge *edge, edgeList) {
        QPointF pos;
        if (edge->sourceNode() == this)
            pos = mapFromItem(edge->destNode(), 0, 0);
        else
            pos = mapFromItem(edge->sourceNode(), 0, 0);
        xvel += pos.x() / weight;
        yvel += pos.y() / weight;
    }
    
    if (qAbs(xvel) < 0.1 && qAbs(yvel) < 0.1)
        xvel = yvel = 0;

    QRectF sceneRect = scene()->sceneRect();
    newPos = pos() + QPointF(xvel, yvel);
    newPos.setX(qMin(qMax(newPos.x(), sceneRect.left() + 10), sceneRect.right() - 10));
    newPos.setY(qMin(qMax(newPos.y(), sceneRect.top() + 10), sceneRect.bottom() - 10));
}

bool QGraphicNode::advance()
{
    if (newPos == pos())
        return false;

    setPos(newPos);
    return true;
}

QRectF QGraphicNode::boundingRect() const
{
    qreal adjust = 2;
    return QRectF(-10 - adjust, -10 - adjust,
                  23 + adjust, 23 + adjust);
}

QPainterPath QGraphicNode::shape() const
{
    QPainterPath path;
    path.addEllipse(-10, -10, 20, 20);
    return path;
}

void QGraphicNode::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *)
{
//    painter->setCompositionMode(QPainter::CompositionMode_SourceOver);
    QRadialGradient gradient(-3, -3, 10);
    if (option->state & QStyle::State_Sunken) {
        gradient.setCenter(3, 3);
        gradient.setFocalPoint(3, 3);
        gradient.setColorAt(1, n_darkColor.light(120));
        gradient.setColorAt(0, n_lightColor.light(120));
    } else {
        gradient.setColorAt(0, n_lightColor);
        gradient.setColorAt(1, n_darkColor);
    }
    painter->setBrush(gradient);
    
    QColor strokeColor = Qt::black ; strokeColor.setAlpha(n_darkColor.alpha()) ;
    painter->setPen(strokeColor);
    painter->drawEllipse(-10, -10, 20, 20);

    // draw text
    QFont font("palatino", QFont::Normal, false) ;
    // font.setBold(true);
    font.setPointSize(9);
    painter->setFont(font);
    painter->setPen(Qt::black);
    painter->drawText(QPointF(-5,2), label) ;
}

QVariant QGraphicNode::itemChange(GraphicsItemChange change, const QVariant &value)
{
    switch (change) {
    case ItemPositionChange:
        foreach (QGraphicEdge *edge, edgeList)
            edge->adjust();
        //graph->itemMoved();
        break;
    default:
        break;
    };

    return QGraphicsItem::itemChange(change, value);
}

void QGraphicNode::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
    update();
    QGraphicsItem::mousePressEvent(event);
}

void QGraphicNode::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{
    update();
    QGraphicsItem::mouseReleaseEvent(event);
}
