/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QParameterSpaceView.cpp
 * 2007-06-13
 * Julien Wintz & Stephane Chau
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# include <QFile>
# include <QCursor>
# include <QAction>
# include <QMenu>
# include <QPainter>
# include <QString> 
# include <QToolTip>

# include <QAxel/QPoint3f.h>
# include <QAxel/QRationalCurve.h>
# include <QAxel/QRationalSurface.h>
# include <QAxel/QBSplineCurve.h>
# include <QAxel/QBSplineSurface.h>
# include <QAxel/ObjectManager.h>

# include <QGui/MainWindow.h>
# include <QGui/QParameterSpaceView.h>
# include <QGui/ManipulatedFrameSetConstraint.h>

# ifndef GL_MULTISAMPLE
# define GL_MULTISAMPLE  0x809D
# endif

using namespace std ;
using namespace QAxel ;

QParameterSpaceView::QParameterSpaceView(QAxelObject * object, QWidget *parent) : QGLViewer(parent, 0, 0) 
{
    this->m_object = object ;
    this->m_nbSubdivisions = 5 ;
    setObjectName("QParameterSpaceView");
}

void QParameterSpaceView::init() 
{
    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,GL_TRUE) ;   

    glEnable(GL_DEPTH_TEST);
    glEnable(GL_MULTISAMPLE);
    glEnable(GL_LINE_SMOOTH) ;
    glEnable(GL_BLEND) ;

    glShadeModel(GL_SMOOTH) ;
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA) ; 
	
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL) ;
    glPolygonOffset(1.0, 1.0) ;
    glEnable(GL_POLYGON_OFFSET_POINT) ;
    glEnable(GL_POLYGON_OFFSET_LINE) ;
    glEnable(GL_POLYGON_OFFSET_FILL) ;

    setSceneRadius(1) ;
    setBackgroundColor(Qt::lightGray) ;

    setCursor(Qt::CrossCursor) ;

    camera()->setType(qglviewer::Camera::ORTHOGRAPHIC) ;
    camera()->setPosition(Vec(0.0, 0.0, 2.5)) ;
    camera()->setUpVector(Vec(0, 1, 0)) ;
    camera()->setViewDirection(Vec(0, 0, -1)) ;

    cameraConstraint = new CameraConstraint(camera()) ;
    cameraConstraint->setRotationConstraintType(AxisPlaneConstraint::FORBIDDEN) ;
    cameraConstraint->setTranslationConstraintType(AxisPlaneConstraint::FORBIDDEN) ;
    camera()->frame()->setConstraint(cameraConstraint) ;
}

void QParameterSpaceView::draw()
{	
    m_width  = qAbs(convertCoordinatesToPoint(width(), height())->x()) ;
    m_height = qAbs(convertCoordinatesToPoint(width(), height())->y()) ;
    drawGrid(m_width, m_height, m_nbSubdivisions) ;
}

void QParameterSpaceView::drawWithNames() 
{            

}

void QParameterSpaceView::endSelection(const QPoint& p)
{
    Q_UNUSED(p);
}

void QParameterSpaceView::keyPressEvent(QKeyEvent *e) 
{
    QGLViewer::keyPressEvent(e) ;
}

void QParameterSpaceView::mousePressEvent(QMouseEvent* e) 
{
    QPoint3f * p = convertClickToPoint(e) ;
    
    float umin = 0.0, umax = 1.0, vmin = 0.0, vmax = 1.0 ;
    
    if(QParametricCurve * curve = qobject_cast<QParametricCurve *>(m_object)) {
	umin = curve->umin() ;
	umax = curve->umax() ;
    }

    if(QParametricSurface * surface = qobject_cast<QParametricSurface *>(m_object)) {
	umin = surface->umin() ;
	umax = surface->umax() ;
	vmin = surface->vmin() ;
	vmax = surface->vmax() ;
    }

    float u = p->x()*(umax-umin)/(2*m_width ) + (umax+umin)/2 ;
    float v = p->y()*(vmax-vmin)/(2*m_height) + (vmax+vmin)/2 ;
    
    m_point = evaluateClick(u, v) ;
    m_point->setColor(Qt::yellow) ;

    emit parameterClicked(u, v) ;

    if(e->button() == Qt::LeftButton)
	ObjectManager::instance()->addObject(m_point) ;

    if(e->button() == Qt::RightButton)
	emit evaluation((QAxelObject *)m_point) ;

    QGLViewer::mousePressEvent(e) ;
}

void QParameterSpaceView::mouseMoveEvent(QMouseEvent* e) 
{
	if(underMouse() && m_point) {

	QPoint3f * p = convertClickToPoint(e) ;
	
	float umin = 0.0, umax = 1.0, vmin = 0.0, vmax = 1.0 ;
	
	if(QParametricCurve * curve = qobject_cast<QParametricCurve *>(m_object)) {
	    umin = curve->umin() ;
	    umax = curve->umax() ;
	}

	if(QParametricSurface * surface = qobject_cast<QParametricSurface *>(m_object)) {
	    umin = surface->umin() ;
	    umax = surface->umax() ;
	    vmin = surface->vmin() ;
	    vmax = surface->vmax() ;
	}
	
	float u = p->x()*(umax-umin)/(2*m_width ) + (umax+umin)/2 ;
	float v = p->y()*(vmax-vmin)/(2*m_height) + (vmax+vmin)/2 ;
    
	QPoint3f * t = evaluateClick(u, v) ;

	m_point->setPosition(t) ;

	QToolTip::showText(e->globalPos(), QString("(%1,%2)").arg(u).arg(v)) ;

	emit(updated()) ;
    }

    QGLViewer::mouseMoveEvent(e) ;
}

void QParameterSpaceView::mouseReleaseEvent(QMouseEvent* e)
{
    if(e->button() == Qt::LeftButton) {
	ObjectManager::instance()->removeObject(m_point) ;
	delete m_point ;
    }

    QGLViewer::mouseReleaseEvent(e) ;
}

void QParameterSpaceView::wheelEvent(QWheelEvent *e)
{
    int numDegrees = e->delta() / 8;
    int numSteps = numDegrees / 15;

    m_nbSubdivisions += numSteps ;
    m_nbSubdivisions = m_nbSubdivisions < 0 ? 0 : m_nbSubdivisions ;

    update() ;
}


void QParameterSpaceView::drawGrid(float width, float height, int nbSubdivisions) 
{
    glDisable(GL_LIGHTING);
    glLineWidth(1.0);
    glColor3f(0.8,0.8,0.8) ;

    glBegin(GL_LINES);
    for (int i=0; i<=nbSubdivisions; ++i) {
	const float posx = width *(2.0*i/nbSubdivisions-1.0);
	const float posy = height*(2.0*i/nbSubdivisions-1.0);
	glVertex3f(posx, -height, 0.0);
	glVertex3f(posx, +height, 0.0);
	glVertex3f(-width, posy, 0.0);
	glVertex3f( width, posy, 0.0);
    }
    glEnd();
    glEnable(GL_LIGHTING);
}

QPoint3f * QParameterSpaceView::convertClickToPoint(QMouseEvent * e)
{
    using qglviewer::Camera ;
    using qglviewer::Vec ;
    
    Vec orig, dir ;
    QPoint p = QPoint(e->x(),e->y()) ; 
    camera()->convertClickToLine(p, orig, dir) ;
    float t = - orig.z/dir.z ;
    Vec poi = orig + t*dir ;
    
    return new QPoint3f(poi.x, poi.y, 0.0) ;
}

QPoint3f * QParameterSpaceView::convertCoordinatesToPoint(int x, int y)
{
    using qglviewer::Camera ;
    using qglviewer::Vec ;
    
    Vec orig, dir ;
    QPoint p = QPoint(x, y) ; 
    camera()->convertClickToLine(p, orig, dir) ;
    float t = - orig.z/dir.z ;
    Vec poi = orig + t*dir ;
    
    return new QPoint3f(poi.x, poi.y, 0.0) ;
}

QPoint3f * QParameterSpaceView::evaluateClick(float u, float v)
{
    if(QRationalCurve * curve = qobject_cast<QRationalCurve *>(m_object))
 	return curve->eval(u) ;
    
    if(QRationalSurface * surface = qobject_cast<QRationalSurface *>(m_object))
 	return surface->eval(u, v) ;
    
    if(QBSplineCurve * curve = qobject_cast<QBSplineCurve *>(m_object))
 	return curve->eval(u) ;
    
    if(QBSplineSurface * surface = qobject_cast<QBSplineSurface *>(m_object))
 	return surface->eval(u, v) ;

    return NULL ;
}
