/*****************************************************************************
 * A x e l
 *****************************************************************************
 * SceneInspectorDock
 * 2006-05-23
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# include <QIcon>
# include <QPixmap>
# include <QTreeView>
# include <QPainter>
# include <QHeaderView>

# include <QGui/Viewer.h>
# include <QGui/SceneInspectorDock.h>
# include <QGui/PropertyEditorDelegate.h>

/*****************************************************************************
 * AxelObjectItem
 ****************************************************************************/

AxelObjectItem::AxelObjectItem(QAxelObject * object, const QList<QVariant> &data, AxelObjectItem *parent)
{
	parentItem = parent;
	itemData = data;
	itemObject = object ;
}

AxelObjectItem::~AxelObjectItem()
{
	qDeleteAll(childItems);
}

void AxelObjectItem::appendChild(AxelObjectItem *item)
{
	childItems.append(item);
}

AxelObjectItem *AxelObjectItem::child(int row)
{
	return childItems.value(row);
}

int AxelObjectItem::childCount() const
{
	return childItems.count();
}

int AxelObjectItem::columnCount() const
{
	return itemData.count();
}

QVariant AxelObjectItem::data(int column) const
{
	return itemData.value(column);
}

AxelObjectItem *AxelObjectItem::parent()
{
	return parentItem;
}

int AxelObjectItem::row() const
{
	if (parentItem)
		return parentItem->childItems.indexOf(const_cast<AxelObjectItem*>(this));

	return 0;
}

/*****************************************************************************
 * AxelObjectModel
 ****************************************************************************/

AxelObjectModel::AxelObjectModel(QList<QAxelObject *> objects, QObject *parent) : QAbstractItemModel(parent)
{
	QList<QVariant> rootData;
	rootData << "Object" << "Type" << "View" ;
	rootItem = new AxelObjectItem(NULL, rootData);
	setupModelData(objects, rootItem);
}

AxelObjectModel::~AxelObjectModel()
{
	delete rootItem;
}

AxelObjectItem * AxelObjectModel::root(void) 
{
	return rootItem ;
}

int AxelObjectModel::columnCount(const QModelIndex &parent) const
{
	if (parent.isValid())
		return static_cast<AxelObjectItem*>(parent.internalPointer())->columnCount();
	else
		return rootItem->columnCount();
}

QVariant AxelObjectModel::data(const QModelIndex &index, int role) const
{
	if (!index.isValid())
		return QVariant();

	if (role != Qt::DisplayRole)
		return QVariant();

	AxelObjectItem *item = static_cast<AxelObjectItem*>(index.internalPointer());

	return item->data(index.column());
}

Qt::ItemFlags AxelObjectModel::flags(const QModelIndex &index) const
{
	if (!index.isValid())
		return Qt::ItemIsEnabled;

	return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
}

QVariant AxelObjectModel::headerData(int section, Qt::Orientation orientation,
							   int role) const
{
	if (orientation == Qt::Horizontal && role == Qt::DisplayRole)
		return rootItem->data(section);

	return QVariant();
}

QModelIndex AxelObjectModel::index(int row, int column, const QModelIndex &parent)
		const
{
	AxelObjectItem *parentItem;

	if (!parent.isValid())
		parentItem = rootItem;
	else
		parentItem = static_cast<AxelObjectItem*>(parent.internalPointer());

	AxelObjectItem *childItem = parentItem->child(row);
	if (childItem)
		return createIndex(row, column, childItem);
	else
		return QModelIndex();
}

QModelIndex AxelObjectModel::parent(const QModelIndex &index) const
{
	if (!index.isValid())
		return QModelIndex();

	AxelObjectItem *childItem = static_cast<AxelObjectItem*>(index.internalPointer());
	AxelObjectItem *parentItem = childItem->parent();

	if (parentItem == rootItem)
		return QModelIndex();

	return createIndex(parentItem->row(), 0, parentItem);
}

int AxelObjectModel::rowCount(const QModelIndex &parent) const
{
	AxelObjectItem *parentItem;

	if (!parent.isValid())
		parentItem = rootItem;
	else
		parentItem = static_cast<AxelObjectItem*>(parent.internalPointer());

	return parentItem->childCount();
}

void AxelObjectModel::setupModelData(QList<QAxelObject *> objects, AxelObjectItem * parent)
{
	QList<AxelObjectItem*> parents ;
	QList<int> indentations ;
	
	parents << parent ;
	
	indentations << 0 ;

	int number = 0;

 	while (number < objects.count()) {
		if(!objects[number]->blacklisted()) {
			bool visible = objects[number]->hidden() ;
			QString name = objects[number]->name() ;
			QString type = objects[number]->type() ;

			QList<QVariant> columnData;
			columnData << name << type << visible ;

			// Append a new item to the current parent's list of children.
			parents.last()->appendChild(new AxelObjectItem(objects[number], columnData, parents.last()));
		}
		number++;
	}
}

/*****************************************************************************
 * AxelObjectItemDelegate
 ****************************************************************************/

AxelObjectItemDelegate::AxelObjectItemDelegate(Viewer * v, AxelObjectModel * m, QObject *parent) : QItemDelegate(parent)
{
	mod = m ;
	this->v = v ;
	viewactive = new QPixmap(":images/16x16/viewactive.png") ;
	viewinactive = new QPixmap(":images/16x16/viewinactive.png") ;
}

void AxelObjectItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
	// if(!d_painter) d_painter = painter ;
    if (index.column() == 2) {
		const QAbstractItemModel *model = index.model() ;
        bool hidden = model->data(index, Qt::DisplayRole).toBool();
        int width = viewactive->width();
        int height = viewactive->height();
        int x = option.rect.x() + (option.rect.width() / 2) - (width / 2) ;
        int y = option.rect.y() + (option.rect.height() / 2) - (height / 2);
        if(hidden) 	painter->drawPixmap(x, y, *viewinactive) ;
        else 		painter->drawPixmap(x, y, *viewactive) ;
    } else QItemDelegate::paint(painter, option, index) ;
}

using namespace std ;

QSize AxelObjectItemDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    if (index.column() == 2) 
        return QSize(16, 16) ;
	else 
		return QItemDelegate::sizeHint(option, index) ;
}

bool AxelObjectItemDelegate::editorEvent(QEvent *event, QAbstractItemModel *model, const QStyleOptionViewItem &option,const QModelIndex &index)
{
	AxelObjectItem * root = mod->root() ;
	AxelObjectItem * item = root->child(index.row()) ;
	QAxelObject * object = item->object() ;
   	bool visible = object->hidden() ;

    if (index.column() == 0) {
		v->selectFromName(object->name()) ;
		v->updateGL() ;
		return true ;
	}

	else if(index.column() == 2) {
	    switch (event->type()) {
	        case QEvent::MouseButtonPress :	            
				object->setHidden(!visible) ;
				v->updateGL() ;
				emit update() ;
	            break ;
	        default:
	            break;
	    }
		return true ;
	}

	return QItemDelegate::editorEvent(event, model, option, index) ;
}

QWidget *AxelObjectItemDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option,
                                    const QModelIndex &index) const
{
    return QItemDelegate::createEditor(parent, option, index);
}

/*****************************************************************************
 * AxelObjectView
 ****************************************************************************/

AxelObjectView::AxelObjectView(AxelObjectModel * model, Viewer * v, QWidget * parent) : QTreeView(parent) 
{
	AxelObjectItemDelegate * delegate = new AxelObjectItemDelegate(v, model, this) ;
    setItemDelegate(delegate) ;
	SceneInspectorDock * dock = dynamic_cast<SceneInspectorDock *>(parent) ;
	connect(delegate, SIGNAL(update()), dock, SLOT(update())) ;
	// connect(delegate, SIGNAL(update), this, SLOT(repaint())) ;
	header()->resizeSections(QHeaderView::Fixed) ;
	header()->resizeSection(2,20) ;
	setModel(model) ;
}

/*****************************************************************************
 * SceneInspectorDock
 ****************************************************************************/

SceneInspectorDock::SceneInspectorDock(Viewer * v, const QString & title, QWidget * parent) : QDockWidget(title, parent) 
{
	this->viewer = v ;
	
	model = new AxelObjectModel(viewer->listObjects()) ;
	tree = new AxelObjectView(model, v, this);
	tree->setAlternatingRowColors(true) ;
	setWidget(tree) ;
}

void SceneInspectorDock::update(void) 
{
	delete model ;
	model = new AxelObjectModel(viewer->listObjects()) ;
	tree->setModel(model) ;
}

void SceneInspectorDock::selectObject(const QModelIndex & index)
{
	QString name = model->data(index,0).toString() ;
	viewer->selectFromName(name) ;
}
